/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/

/*
main internal include file for FSDB
*/

#ifndef __UND_FSDB_H__
#define __UND_FSDB_H__

// this code comes from the IBPP library:
// IBPP is written for 32 bits systems or higher.
// The standard type 'int' is assumed to be at least 32 bits.
// And the standard type 'short' is assumed to be exactly 16 bits.
// Everywhere possible, where the exact size of an integer does not matter,
// the standard type 'int' is used. And where an exact integer size is required
// the standard exact precision types definitions of C 99 standard are used.

#if defined(_MSC_VER) || defined(__DMC__) || defined(__BCPLUSPLUS__)
// C99 7.18.1.1 Exact-width integer types (only those used by IBPP)
#if defined(_MSC_VER) && (_MSC_VER < 1300)	// MSVC 6 should be < 1300
	typedef short int16_t;
	typedef int int32_t;
	typedef unsigned int uint32_t;
#else
	typedef __int16 int16_t;
	typedef __int32 int32_t;
	typedef unsigned __int32 uint32_t;
#endif
	typedef __int64 int64_t;
#else
	#include <stdint.h>			// C99 (7.18) integer types definitions
#endif

/*! \namespace FSDB
 * Handle database access in a portable and multidatabase way
 */
namespace FSDB {

	/// The possible kinds of recordset objects
	enum RecordsetType { 
		rtReadOnlyForwardOnly,				/*!< read only, can only move forwards */
		rtReadOnlyRepeatableForwardOnly,	/*!< read only, can only move forwards but can move back to the first row */
		rtReadWriteForwardOnly,				/*!< read write, can only move forwards */
		rtReadWrite							/*!< read write, can move forwards and backwards */
	};

}

#endif
