/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/


#include "data_error.h"

CDataErrorException::CDataErrorException( const string &errMsg, ErrorCause ec ) : runtime_error( errMsg ) {

	m_ErrorCode = ec;
}

CDataErrorException::CDataErrorException( const char *errMsg, ErrorCause ec ) : runtime_error( errMsg ) {

	m_ErrorCode = ec;
}

CDataErrorException::ErrorCause CDataErrorException::GetErrorCause() {
	return m_ErrorCode;
}

bool CDataErrorException::ConvertFirebirdError( long gdscode, ErrorCause& ec ) {

	bool retVal = false;

	switch( gdscode ) {
		case 335544558:
			ec = ecCheck;
			retVal = true;
			break;
		case 335544665:
			ec = ecUnique;
			retVal = true;
			break;
		case 335544466:
			ec = ecReferentialIntegrity;
			retVal = true;
			break;
		case 335544721:
        case 335544375:
			ec = ecServerNotFound;
			retVal = true;
			break;
		case 335544344:
			ec = ecDatabaseNotFound;
			retVal = true;
			break;
		case 335544472:
			ec = ecWrongUsernameOrPassword;
			retVal = true;
			break;
		case 335544517:
			ec = ecException;
			retVal = true;
			break;
	}

	return retVal;
}

CDataErrorException CDataErrorException::IBPPLogic2CDataError( IBPP::LogicException &eIn, const string &additionalInfo ) {

	CDataErrorException::ErrorCause ec = CDataErrorException::ecUnspecified;
	std::string msg;

	msg = eIn.what() + std::string("\n\nORIGIN: ") + eIn.Origin();
    if( !additionalInfo.empty() ) {
        msg += "\n\nADDITIONAL INFO: " + additionalInfo;
    }

	CDataErrorException eOut( msg, ec );
	return eOut;
}

CDataErrorException CDataErrorException::IBPPSql2CDataError( IBPP::SQLException &eIn ) {

	CDataErrorException::ErrorCause ec;
	std::string msg;

	msg = eIn.what() + std::string("\n\nORIGIN: ") + eIn.Origin();
	if( !CDataErrorException::ConvertFirebirdError( eIn.EngineCode(), ec )  ) {
		ec = CDataErrorException::ecUnspecified;
	}

	CDataErrorException eOut( msg, ec );
	return eOut;
}

