/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/


#ifndef __FSDB_RECORDSET_FO_RW_H__
#define __FSDB_RECORDSET_FO_RW_H__

#include "recordset_fo_ro.h"

namespace FSDB {

	/// A forward-only, read-write recordset
	class Recordset_FO_RW : public Recordset_FO_RO {

	public:

		Recordset_FO_RW( RecordsetType rt, DbInterfaceBase *DbInterface ) : 
					Recordset_FO_RO( rt, DbInterface ),
					row(NULL) {}
		~Recordset_FO_RW();
		
		/// open a read-write recordset.
		/// receives the sql string, the name of the underlying table and its primary key name.
		void Open( const std::string &sql, const std::string &tableName = "", const std::string &primaryKeyName = "" );
		void Open( const std::string &sql, Parameters params, const std::string &tableName = "", const std::string &primaryKeyName = "" );
		void Close();

		void MoveNext();

		void Edit();
		void AddNew();
		void Update();
		void CancelUpdate();

        void Delete();

		Variant& operator[]( const std::string& column );

	protected:

		/// possible states of the recordset
		enum CurrentState {
			csNormal, /*!< normal */
			csEdit, /*!< editing an existing row */
			csAddNew, /*!< appending a new row */
			csDeleted /*!< the current row has been deleted */
		};

		/// current state
		CurrentState m_CurrentState;

		void ReleaseMemory();

		/// number of columns in the recordset
		int nColumns;

		/// rowbuffer: array that contains all the columns for a row
		Variant *row;

        /// throw an exception if the current row has been deleted
        void CheckDeleted();

	};

}

#endif

