/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _FS_DATE_PICKER_H_
#define _FS_DATE_PICKER_H_

#include <wx/calctrl.h>
#include <wx/datectrl.h>    // we need the style flags
#include <wx/combo.h>
#include <wx/textctrl.h>


/*
this file defines two classes used for date input:

fsDatePickerCtrl and
fsDatePickerTextCtrl

the former looks like a combo and it can pop up a calendar control
the latter is a custom text control for date entry: I use it because fsDatePickerCtrl has problems in many situations

the text control of the combo baheves exactly like fsDatePickerTextCtrl and the two controls have the same interface
*/

class WXDLLIMPEXP_FWD_ADV fsCalendarComboPopup;

/*
This class is similar to a generic combo wdDatePickedCtrl showing a native wxCalendarCtrl
It is directly derived from wxComboCtrl because with this solution tab navigation is not broken when opening the popup
The only recognized style is wxDP_SHOWCENTURY
*/

class WXDLLIMPEXP_ADV fsDatePickerCtrl : public wxComboCtrl
{
public:
    // creating the control
    fsDatePickerCtrl() { Init(); }
    virtual ~fsDatePickerCtrl();
    fsDatePickerCtrl(wxWindow *parent,
                            wxWindowID id,
                            const wxDateTime& date = wxDefaultDateTime,
                            const wxPoint& pos = wxDefaultPosition,
                            const wxSize& size = wxDefaultSize,
                            long style = wxDP_SHOWCENTURY,
                            const wxValidator& validator = wxDefaultValidator)
    {
        Init();

        (void)Create(parent, id, date, pos, size, style, validator);
    }

    bool Create(wxWindow *parent,
                wxWindowID id,
                const wxDateTime& date = wxDefaultDateTime,
                const wxPoint& pos = wxDefaultPosition,
                const wxSize& size = wxDefaultSize,
                long style = wxDP_SHOWCENTURY,
                const wxValidator& validator = wxDefaultValidator);

    // these methods use a wxDateTime parameter
    // they cannot be called GetValue() and SetValue() because those names are defined by wxComboCtrl
    // and they use a wxString representation of the date (empty string for no date)
    void SetValueDT(const wxDateTime& date);
    wxDateTime GetValueDT() const;

    // implementation only from now on
    // -------------------------------

    // overridden base class methods
    virtual bool Destroy();

protected:
    virtual wxSize DoGetBestSize() const;

private:
    void Init();

    void OnText(wxCommandEvent &event);
    void OnChar(wxKeyEvent &event);
    void OnKeyDown(wxKeyEvent &event);

    fsCalendarComboPopup* m_popup;

    DECLARE_EVENT_TABLE()
    DECLARE_DYNAMIC_CLASS(fsDatePickerCtrl);
};


class WXDLLIMPEXP_ADV fsDatePickerTextCtrl : public wxTextCtrl
{
public:
    // creating the control
    fsDatePickerTextCtrl() { Init(); }
    virtual ~fsDatePickerTextCtrl();
    fsDatePickerTextCtrl(wxWindow *parent,
                            wxWindowID id,
                            const wxDateTime& date = wxDefaultDateTime,
                            const wxPoint& pos = wxDefaultPosition,
                            const wxSize& size = wxDefaultSize,
                            long style = wxDP_SHOWCENTURY,
                            const wxValidator& validator = wxDefaultValidator)
    {
        Init();

        (void)Create(parent, id, date, pos, size, style, validator);
    }

    bool Create(wxWindow *parent,
                wxWindowID id,
                const wxDateTime& date = wxDefaultDateTime,
                const wxPoint& pos = wxDefaultPosition,
                const wxSize& size = wxDefaultSize,
                long style = wxDP_SHOWCENTURY,
                const wxValidator& validator = wxDefaultValidator);

    // these methods use a wxDateTime parameter
    // they cannot be called GetValue() and SetValue() because those names are defined by wxTextCtrl
    // and they use a wxString representation of the date (empty string for no date)
    void SetValueDT(const wxDateTime& date);
    wxDateTime GetValueDT() const;

    // implementation only from now on
    // -------------------------------

protected:
    virtual wxSize DoGetBestSize() const;

    void Init();

    void SetFormat(const wxString& fmt);
    // it expands "%x" format and changes %y to %Y if wxDP_SHOWCENTURY flag
    // is given. If the locale format can't be easily analyzed (e.g. when
    // the month is given as a name, not number), "%x" is returned
    wxString GetLocaleDateFormat();
    void SetSeparator( wxString &sdate );


    void OnChar(wxKeyEvent &event);
    void OnKeyDown(wxKeyEvent &event);
    void OnKillFocus(wxFocusEvent &event);

    wxString        m_format;
    wxChar          m_separator;  // the current date separator
    static const wxChar    m_invalidSeparator = '1';

    DECLARE_EVENT_TABLE()
    DECLARE_DYNAMIC_CLASS(fsDatePickerTextCtrl);
};


#endif

