/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif


#include "app_settings.h"
#include "vvvpapp.h"
#include "metadata.h"
#include "utils.h"

#include "wx/config.h"

CAppSettings::CAppSettings(void)
{
	m_fileHistory = new wxFileHistory();
}

CAppSettings::~CAppSettings(void)
{
	delete m_fileHistory;
}

CAppSettings::CAppSettings( const CAppSettings& as ) {
	Copy( as );
}

void CAppSettings::operator=(const CAppSettings& as) {
	Copy( as );
}

void CAppSettings::Copy( const CAppSettings& as ) {
	// this function is only used to transfer data to the Settings dialog
	// no need to copy data that is not used in that dialog

	m_ReopenLastUsedCatalog = as.m_ReopenLastUsedCatalog;
	m_BeepTime = as.m_BeepTime;
	m_ForceEnglishLanguage = as.m_ForceEnglishLanguage;
	m_ThumbnailSize = as.m_ThumbnailSize;
	m_UseJpegPreview = as.m_UseJpegPreview;
	m_ThumbnailViewSmallSize = as.m_ThumbnailViewSmallSize;
	m_ThumbnailViewMediumSize = as.m_ThumbnailViewMediumSize;
	m_ThumbnailViewLargeSize = as.m_ThumbnailViewLargeSize;
	m_CurrentThumbnailViewSize = as.m_CurrentThumbnailViewSize;
	m_CurrentView = as.m_CurrentView;

//	do not copy m_fileHistory because it causes a call to delete as soon as the second object is destroyed
//	the variable is then still used causing an exception
//	m_fileHistory = as.m_fileHistory;

	m_EnableColorManagement = as.m_EnableColorManagement;
	m_MonitorProfile = as.m_MonitorProfile;
	m_FileProfileName = as.m_FileProfileName;

	dbConnectToServer = as.dbConnectToServer;
	dbServerName = as.dbServerName;
	dbUserName = as.dbUserName;
	dbPassword = as.dbPassword;

	mf_Maximized = as.mf_Maximized;
	mf_XPosition = as.mf_XPosition;
	mf_YPosition = as.mf_YPosition;
	mf_Width = as.mf_Width;
	mf_Height = as.mf_Height;
	mf_SashPosition = as.mf_SashPosition;
	m_ListviewColWidth = as.m_ListviewColWidth;

	m_CatalogPath = as.m_CatalogPath;
	m_CatalogRemovableLocation = as.m_CatalogRemovableLocation;
	m_CatalogFixedLocation = as.m_CatalogFixedLocation;
	m_LastFixedLocation = as.m_LastFixedLocation;

	m_AddVirtualFoldersLastUsed = as.m_AddVirtualFoldersLastUsed;

	m_ExtractPreviewWhenCopyingRawFiles = as.m_ExtractPreviewWhenCopyingRawFiles;
	m_CopyFilesDestinationPath = as.m_CopyFilesDestinationPath;

	m_ExternalPrograms = as.m_ExternalPrograms;
	m_ExternalProgramsTempPath = as.m_ExternalProgramsTempPath;

    m_TimelineStartDate = as.m_TimelineStartDate;
    m_TimelineEndDate = as.m_TimelineEndDate;
    m_TimelineThreshold = as.m_TimelineThreshold;
}

void CAppSettings::GetConnectionData( wxString& serverName, wxString& userName, wxString& password ) {

	if( this->dbConnectToServer ) {
		serverName = this->dbServerName;
		userName = this->dbUserName;
		password = this->dbPassword;
	}
	else {
		// parameters for embedded server
		serverName = wxEmptyString;
		userName = wxT("SYSDBA");
		password = wxT("masterkey");
	}
}

void CAppSettings::Load() {
	int itmp;
    wxString s;

	wxConfigBase *pConfig = wxConfigBase::Get();
	
	// read the main frame settings
	pConfig->SetPath(wxT("/Mainframe/Layout"));
	mf_XPosition = pConfig->Read(wxT("x"), 50);
	mf_YPosition = pConfig->Read(wxT("y"), 50);
	mf_Width = pConfig->Read(wxT("w"), 600);
	mf_Height = pConfig->Read(wxT("h"), 400);
	pConfig->Read( wxT("Maximized"), &mf_Maximized, false );
	mf_SashPosition = pConfig->Read( wxT("SashPosition"), 200 );

	// read program settings
	pConfig->SetPath(wxT("/Settings"));
	// if there is a settings file name we are running a portable version of this application, so we do not reopen catalogs at startup by default
	bool defVal = true;
	if( !wxGetApp().GetSettingsFileName().empty() ) defVal = false;
	pConfig->Read( wxT("ReopenCatalog"), &m_ReopenLastUsedCatalog, defVal );
	pConfig->Read( wxT("ForceEnglish"), &m_ForceEnglishLanguage, false );
	pConfig->Read( wxT("LongTaskBeepTime"), &m_BeepTime, 5 );
	pConfig->Read( wxT("ThumbnailSize"), &m_ThumbnailSize, 256 );
	pConfig->Read( wxT("UseJpegPreview"), &m_UseJpegPreview, false );
	pConfig->Read( wxT("ThumbnailViewSmallSize"), &m_ThumbnailViewSmallSize, 64 );
	pConfig->Read( wxT("ThumbnailViewMediumSize"), &m_ThumbnailViewMediumSize, 128 );
	pConfig->Read( wxT("ThumbnailViewLargeSize"), &m_ThumbnailViewLargeSize, 256 );
	pConfig->Read( wxT("CurrentThumbnailViewSize"), &itmp, tsMediumThumbnails );
	m_CurrentThumbnailViewSize = (ThumbnailSize) itmp;
	pConfig->Read( wxT("CurrentView"), &itmp, cvRemovable );
	m_CurrentView = (CurrentView) itmp;

	pConfig->Read( wxT("EnableColorManagement"), &m_EnableColorManagement, true );
	pConfig->Read( wxT("MonitorProfileKind"), &itmp, mp_sRGB );
	m_MonitorProfile = (MonitorProfile) itmp;
	m_FileProfileName = pConfig->Read( wxT("FileProfileName"), wxT("") );

	m_CatalogPath = pConfig->Read( wxT("CatalogPath"), wxT("") );
	pConfig->Read( wxT("CatalogRemovableLocation"), &m_CatalogRemovableLocation, false );
	pConfig->Read( wxT("CatalogFixedLocation"), &m_CatalogFixedLocation, false );
	m_LastFixedLocation = pConfig->Read( wxT("LastFixedLocation"), wxT("") );

	m_CopyFilesDestinationPath = pConfig->Read( wxT("CopyFilesDestinationPath"), wxT("") );
	pConfig->Read( wxT("ExtractPreviewWhenCopyingRawFiles"), &m_ExtractPreviewWhenCopyingRawFiles, false );

	m_ExternalProgramsTempPath = pConfig->Read( wxT("ExternalProgramsTempPath"), wxT("") );

	// read the MRU files list
	pConfig->SetPath(wxT("/Settings/MRU"));
	m_fileHistory->Load( *pConfig );

	pConfig->SetPath(wxT("/Settings/DatabaseServer"));
	pConfig->Read( wxT("ConnectToServer"), &dbConnectToServer, false );
	dbServerName = pConfig->Read( wxT("ServerName"), wxEmptyString );
	dbUserName = pConfig->Read( wxT("UserName"), wxT("SYSDBA") );
	wxString pwd = pConfig->Read( wxT("Password"), wxT("m1a1s1t1e1r1k1e1y1i1m1p1o1s1s1i1b1l1e1d1e1f1a1u1l1t1p1a1s1s1w1o1r1d") );
	if( pwd == wxT("m1a1s1t1e1r1k1e1y1i1m1p1o1s1s1i1b1l1e1d1e1f1a1u1l1t1p1a1s1s1w1o1r1d") ) 
		pwd = wxT("masterkey");	// default password for Firebird servers
	else {
		if( pwd != wxEmptyString )	pwd = CUtils::Encrypt( pwd );
	}
	dbPassword = pwd;

	// read the listview columns width
	m_ListviewColWidth.clear();
	int k = 0;
	pConfig->SetPath(wxT("/Mainframe/Layout/ListView"));
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 150) );	// file name
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 60) );	// file size
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 130) );	// file date
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 200) );	// description
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 130) );	// tag: image timestamp
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 60) );	// tag: make
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 60) );	// tag: model
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 60) );	// tag: software
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 60) );	// tag: exp time
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: aperture
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: max aperture
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: exp bias
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 60) );	// tag: flash
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 80) );	// tag: focal length
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 80) );	// tag: 35mm eq
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: ISO
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: exp mode
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: metering mode
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: image quality
	m_ListviewColWidth.push_back( pConfig->Read(wxT("cw") + CUtils::long2string(k++), 50) );	// tag: white balance

	pConfig->SetPath(wxT("/Settings/AddToVirtualFolderMRU"));
	m_AddVirtualFoldersLastUsed.clear();
	int nElements;
	pConfig->Read( wxT("NFolders"), &nElements, 0 );
	for( k = 0; k < nElements; k++ ) {
		pConfig->SetPath(wxT("/Settings/AddToVirtualFolderMRU/Folder") + CUtils::long2string(k) );
		wxArrayString as;
		LoadArrayString( pConfig, as );
		m_AddVirtualFoldersLastUsed.push_back( as );
	}

	pConfig->SetPath(wxT("/Settings/Timeline"));
	pConfig->Read( wxT("StartDate"), &s, "" );
    if( s.empty() ) {
        m_TimelineStartDate = wxInvalidDateTime;
    }
    else {
        m_TimelineStartDate.ParseFormat( s, "%Y%m%d" );
    }
	pConfig->Read( wxT("EndDate"), &s, "" );
    if( s.empty() ) {
        m_TimelineEndDate = wxInvalidDateTime;
    }
    else {
        m_TimelineEndDate.ParseFormat( s, "%Y%m%d" );
    }
	pConfig->Read( wxT("Threshold"), &m_TimelineThreshold, 0 );

	// read the external programs
	m_ExternalPrograms.Load();
}

void CAppSettings::Save() {
	int itmp, k;
    wxString s;

	wxConfigBase *pConfig = wxConfigBase::Get();
	if ( pConfig == NULL )
		return;

	// save the frame position
	pConfig->SetPath(wxT("/Mainframe/Layout"));
	if( !mf_Maximized ) {
		pConfig->Write(wxT("x"), (long) mf_XPosition);
		pConfig->Write(wxT("y"), (long) mf_YPosition);
		pConfig->Write(wxT("w"), (long) mf_Width);
		pConfig->Write(wxT("h"), (long) mf_Height);
	}
	pConfig->Write(wxT("Maximized"), mf_Maximized );
	pConfig->Write(wxT("SashPosition"), mf_SashPosition );

	// saves program settings
	pConfig->SetPath(wxT("/Settings"));
	pConfig->Write( wxT("ReopenCatalog"), m_ReopenLastUsedCatalog );
	pConfig->Write( wxT("ForceEnglish"), m_ForceEnglishLanguage );
	pConfig->Write( wxT("LongTaskBeepTime"), m_BeepTime );
	pConfig->Write( wxT("ThumbnailSize"), m_ThumbnailSize );
	pConfig->Write( wxT("UseJpegPreview"), m_UseJpegPreview );
	pConfig->Write( wxT("ThumbnailViewSmallSize"), m_ThumbnailViewSmallSize );
	pConfig->Write( wxT("ThumbnailViewMediumSize"), m_ThumbnailViewMediumSize );
	pConfig->Write( wxT("ThumbnailViewLargeSize"), m_ThumbnailViewLargeSize );
	itmp = m_CurrentThumbnailViewSize;
	pConfig->Write( wxT("CurrentThumbnailViewSize"), itmp );
	itmp = m_CurrentView;
	pConfig->Write( wxT("CurrentView"), itmp );

	pConfig->Write( wxT("EnableColorManagement"), m_EnableColorManagement );
	itmp = m_MonitorProfile;
	pConfig->Write( wxT("MonitorProfileKind"), itmp );
	pConfig->Write( wxT("FileProfileName"), m_FileProfileName );

	pConfig->Write( wxT("CatalogPath"), m_CatalogPath );
	pConfig->Write(wxT("CatalogRemovableLocation"), m_CatalogRemovableLocation );
	pConfig->Write(wxT("CatalogFixedLocation"), m_CatalogFixedLocation );
	pConfig->Write( wxT("LastFixedLocation"), m_LastFixedLocation );
	pConfig->Write( wxT("CopyFilesDestinationPath"), m_CopyFilesDestinationPath );
	pConfig->Write(wxT("ExtractPreviewWhenCopyingRawFiles"), m_ExtractPreviewWhenCopyingRawFiles );
	pConfig->Write( wxT("ExternalProgramsTempPath"), m_ExternalProgramsTempPath );

	// save the MRU files list
	pConfig->SetPath(wxT("/Settings/MRU"));
	m_fileHistory->Save( *pConfig );

	pConfig->SetPath(wxT("/Settings/DatabaseServer"));
	pConfig->Write( wxT("ConnectToServer"), dbConnectToServer );
	pConfig->Write( wxT("ServerName"),  dbServerName );
	pConfig->Write( wxT("UserName"),  dbUserName );
	wxString pwd = dbPassword;
	if( pwd != wxEmptyString ) pwd = CUtils::Encrypt( pwd );
	pConfig->Write( wxT("Password"),  pwd );

	pConfig->SetPath(wxT("/Mainframe/Layout/ListView"));
	for( k = 0; k < ListViewColumnsNumber(); k++ )
		pConfig->Write(wxT("cw") + CUtils::long2string(k), (long) m_ListviewColWidth[k]);

	pConfig->SetPath(wxT("/Settings/AddToVirtualFolderMRU"));
	pConfig->Write( wxT("NFolders"), (long) m_AddVirtualFoldersLastUsed.size() );
	for( k = 0; k < (int) m_AddVirtualFoldersLastUsed.size(); k++ ) {
		pConfig->SetPath(wxT("/Settings/AddToVirtualFolderMRU/Folder") + CUtils::long2string(k) );
		SaveArrayString( pConfig, m_AddVirtualFoldersLastUsed[k] );
	}

	pConfig->SetPath(wxT("/Settings/Timeline"));
    s = m_TimelineStartDate.IsValid() ? m_TimelineStartDate.Format( "%Y%m%d" ) : "";
	pConfig->Write( "StartDate",  s );
    s = m_TimelineEndDate.IsValid() ? m_TimelineEndDate.Format( "%Y%m%d" ) : "";
	pConfig->Write( "EndDate",  s );
	pConfig->Write( "Threshold", m_TimelineThreshold );

	// save the external programs
	m_ExternalPrograms.Save();
}

int CAppSettings::ListViewColumnsNumber() {

	// the columns number is the sum of the columns used to show file data
	// and the columns used to show metadata
	return CMetadata::GetTagsNumber() + 4;
}

void CAppSettings::SaveArrayString( wxConfigBase *pConfig, wxArrayString& as )
{
	pConfig->Write( wxT("as_count"), (long) as.GetCount() );
	for( size_t k = 0; k < as.GetCount(); k++ ) {
		pConfig->Write(wxT("as") + CUtils::long2string(k), as[k]);
	}
}

void CAppSettings::LoadArrayString( wxConfigBase *pConfig, wxArrayString& as )
{
	as.Clear();

	int nElements;
	pConfig->Read( wxT("as_count"), &nElements, 0 );
	for( int k = 0; k < nElements; k++ ) {
		wxString stmp = pConfig->Read( wxT("as") + CUtils::long2string(k), wxEmptyString );
		as.Add( stmp );
	}
}

