/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    aint with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        catalog_images.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     31/12/2008 10:35:36
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
////@end includes

#include "catalog_images.h"
#include "catalog_images_functions.h"
#include "utils.h"
#include "fixed_locations.h"
#include "app_settings.h"
#include "vvvpapp.h"
#include "long_task_beep.h"
#include "db_sequences.h"
#include "sqlstrings.h"
#include "metadata.h"
#include "sha1wrapper.h"
#include "long_text_question.h"
#include "compare_folders.h"

#include <exiv2/image.hpp>
#include <exiv2/exif.hpp>

#include <wx/dirdlg.h>
#include <wx/dir.h>
#include <wx/image.h>
#include <wx/bitmap.h>
#include <wx/mstream.h>
#include <wx/file.h>

#ifdef __WXMSW__
#include "windows_specific.h"
#endif

using namespace FSDB;

////@begin XPM images
////@end XPM images


/*!
 * CDialogCatalogImages type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CDialogCatalogImages, wxDialog )


/*!
 * CDialogCatalogImages event table definition
 */

BEGIN_EVENT_TABLE( CDialogCatalogImages, wxDialog )

////@begin CDialogCatalogImages event table entries
    EVT_UPDATE_UI( ID_PANEL_REMOVABLE, CDialogCatalogImages::OnPanelRemovableUpdate )

    EVT_BUTTON( ID_VOLUME_BROWSE, CDialogCatalogImages::OnVolumeBrowseClick )

#if defined(__WXMSW__)
    EVT_BUTTON( ID_GET_VOLUME_NAME, CDialogCatalogImages::OnGetVolumeNameClick )
#endif

    EVT_BUTTON( ID_FOLDER_BROWSE, CDialogCatalogImages::OnFolderBrowseClick )

    EVT_CHECKBOX( ID_HANDLE_FIXED, CDialogCatalogImages::OnHandleFixedClick )

    EVT_UPDATE_UI( ID_PANEL_FIXED, CDialogCatalogImages::OnPanelFixedUpdate )

#if defined(__WXMSW__) || defined(__WXMAC__)
    EVT_CHOICE( ID_CHOICE_FIXED_LOCATION, CDialogCatalogImages::OnChoiceFixedLocationSelected )
#endif

#if defined(__WXGTK__)
    EVT_COMBOBOX( ID_COMBO_FIXED_LOCATION, CDialogCatalogImages::OnChoiceFixedLocationSelected )
#endif

    EVT_BUTTON( ID_FIXED_FOLDER_BROWSE, CDialogCatalogImages::OnFixedFolderBrowseClick )

    EVT_BUTTON( wxID_HELP, CDialogCatalogImages::OnHelpClick )

    EVT_BUTTON( ID_BUTTON_CATALOG, CDialogCatalogImages::OnButtonCatalogClick )

////@end CDialogCatalogImages event table entries

END_EVENT_TABLE()


/*!
 * CDialogCatalogImages constructors
 */

CDialogCatalogImages::CDialogCatalogImages()
{
    Init();
}

CDialogCatalogImages::CDialogCatalogImages( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CDialogCatalogImages creator
 */

bool CDialogCatalogImages::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CDialogCatalogImages creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CDialogCatalogImages creation

	CAppSettings& as = wxGetApp().GetAppSettings();
	m_VolumePath->SetValue( as.m_CatalogPath );
	m_FolderPathRemovable->SetValue( m_VolumePath->GetValue() );

	m_WindowPosition.SetWindow( this );
	m_WindowPosition.RestorePosition();

	// fill the combo box
	CFixedLocations& fl = wxGetApp().GetFixedLocations();
	for( int k = 0; k < fl.size(); k++ ) {
		m_FixedLocationsCombo->Append( fl[k].GetName() + wxT(" (") + fl[k].GetPath() + wxT(") ") + fl[k].GetDescription() );
	}
	// select the last used item
	if( !as.m_LastFixedLocation.empty() ) {
		for( int k = 0; k < fl.size(); k++ ) {
			if( fl[k].GetName() == as.m_LastFixedLocation ) {
				m_FixedLocationsCombo->Select( k );
				m_CurrentFixedLocationComboIndex = k;
				m_FolderPathFixed->SetValue( fl[m_CurrentFixedLocationComboIndex].GetPath() );
			}
		}
	}

	m_ChkHandleRemovable->SetValue( as.m_CatalogRemovableLocation );
	m_ChkHandleFixed->SetValue( as.m_CatalogFixedLocation );

	m_LastCatalogedCtrl->SetLabel( wxEmptyString );

	// check that there are items in the combo if fixed locations catalog is enabled
	if( m_ChkHandleFixed->IsChecked() ) {
		if( fl.size() == 0 ) {
			// the user wants to catalog fixed locations but he has not defined any
			ShowWarningMissingFixedLocation();
		}
	}

	return true;
}


/*!
 * CDialogCatalogImages destructor
 */

CDialogCatalogImages::~CDialogCatalogImages()
{
////@begin CDialogCatalogImages destruction
////@end CDialogCatalogImages destruction

	CFixedLocations& fl = wxGetApp().GetFixedLocations();
	CAppSettings& as = wxGetApp().GetAppSettings();
	as.m_CatalogPath = m_VolumePath->GetValue();
	as.m_CatalogRemovableLocation = m_ChkHandleRemovable->GetValue();
	as.m_CatalogFixedLocation = m_ChkHandleFixed->GetValue();

	if( m_CurrentFixedLocationComboIndex >= 0 ) {
		as.m_LastFixedLocation = fl[m_CurrentFixedLocationComboIndex].GetName();
	}

	m_WindowPosition.SavePosition();
}


/*!
 * Member initialisation
 */

void CDialogCatalogImages::Init()
{
////@begin CDialogCatalogImages member initialisation
    m_MainSizer = NULL;
    m_ChkHandleRemovable = NULL;
    m_PanelRemovable = NULL;
    m_VolumePath = NULL;
    m_VolumeName = NULL;
    m_FolderPathRemovable = NULL;
    m_ChkHandleFixed = NULL;
    m_PanelFixed = NULL;
#if defined(__WXMSW__) || defined(__WXMAC__)
    m_FixedLocationsCombo = NULL;
#endif
#if defined(__WXGTK__)
    m_FixedLocationsCombo = NULL;
#endif
    m_FolderPathFixed = NULL;
    m_HelpButton = NULL;
    m_CatalogButton = NULL;
    m_CloseButton = NULL;
    m_LastCatalogedCtrl = NULL;
    m_ProgressBarCtrl = NULL;
////@end CDialogCatalogImages member initialisation

	cataloging = false;
	cancelCatalogTask = false;

	m_CurrentFixedLocationComboIndex = -1;
	m_DatabaseChanged = false;
}


/*!
 * Control creation for CDialogCatalogImages
 */

void CDialogCatalogImages::CreateControls()
{    
////@begin CDialogCatalogImages content construction
    CDialogCatalogImages* itemDialog1 = this;

    m_MainSizer = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(m_MainSizer);

    m_ChkHandleRemovable = new wxCheckBox( itemDialog1, ID_HANDLE_REMOVABLE, _("Handle removable volumes"), wxDefaultPosition, wxDefaultSize, 0 );
    m_ChkHandleRemovable->SetValue(false);
    m_MainSizer->Add(m_ChkHandleRemovable, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT|wxTOP, 5);

    m_PanelRemovable = new wxPanel( itemDialog1, ID_PANEL_REMOVABLE, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    m_MainSizer->Add(m_PanelRemovable, 0, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxVERTICAL);
    m_PanelRemovable->SetSizer(itemBoxSizer5);

    wxStaticText* itemStaticText6 = new wxStaticText( m_PanelRemovable, wxID_STATIC, _("Enter or select the volume to catalog"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemStaticText6, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT|wxTOP, 5);

    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer5->Add(itemBoxSizer7, 0, wxGROW, 5);

    m_VolumePath = new wxTextCtrl( m_PanelRemovable, ID_VOLUME_PATH, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer7->Add(m_VolumePath, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton9 = new wxButton( m_PanelRemovable, ID_VOLUME_BROWSE, _("..."), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT );
    itemBoxSizer7->Add(itemButton9, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP|wxBOTTOM, 5);

    wxStaticText* itemStaticText10 = new wxStaticText( m_PanelRemovable, wxID_STATIC, _("Volume name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemStaticText10, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT, 5);

    wxBoxSizer* itemBoxSizer11 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer5->Add(itemBoxSizer11, 0, wxGROW, 5);

    m_VolumeName = new wxTextCtrl( m_PanelRemovable, ID_VOLUME_NAME, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer11->Add(m_VolumeName, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

#if defined(__WXMSW__)
    wxButton* itemButton13 = new wxButton( m_PanelRemovable, ID_GET_VOLUME_NAME, _("Get volume name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer11->Add(itemButton13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

#endif

    wxStaticText* itemStaticText14 = new wxStaticText( m_PanelRemovable, wxID_STATIC, _("Enter or select the folder to catalog"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemStaticText14, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT, 5);

    wxBoxSizer* itemBoxSizer15 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer5->Add(itemBoxSizer15, 0, wxGROW, 5);

    m_FolderPathRemovable = new wxTextCtrl( m_PanelRemovable, ID_REM_FOLDER_PATH, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer15->Add(m_FolderPathRemovable, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton17 = new wxButton( m_PanelRemovable, ID_FOLDER_BROWSE, _("..."), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT );
    itemBoxSizer15->Add(itemButton17, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP|wxBOTTOM, 5);

    m_ChkHandleFixed = new wxCheckBox( itemDialog1, ID_HANDLE_FIXED, _("Handle fixed volumes"), wxDefaultPosition, wxDefaultSize, 0 );
    m_ChkHandleFixed->SetValue(false);
    m_MainSizer->Add(m_ChkHandleFixed, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT|wxTOP, 5);

    m_PanelFixed = new wxPanel( itemDialog1, ID_PANEL_FIXED, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    m_MainSizer->Add(m_PanelFixed, 0, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer20 = new wxBoxSizer(wxVERTICAL);
    m_PanelFixed->SetSizer(itemBoxSizer20);

    wxStaticText* itemStaticText21 = new wxStaticText( m_PanelFixed, wxID_STATIC, _("Select the fixed location"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer20->Add(itemStaticText21, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT|wxTOP, 5);

#if defined(__WXMSW__) || defined(__WXMAC__)
    wxArrayString m_FixedLocationsComboStrings;
    m_FixedLocationsCombo = new wxChoice( m_PanelFixed, ID_CHOICE_FIXED_LOCATION, wxDefaultPosition, wxDefaultSize, m_FixedLocationsComboStrings, 0 );
    itemBoxSizer20->Add(m_FixedLocationsCombo, 0, wxGROW|wxALL, 5);

#endif

#if defined(__WXGTK__)
    wxArrayString m_FixedLocationsComboStrings;
    m_FixedLocationsCombo = new wxComboBox( m_PanelFixed, ID_COMBO_FIXED_LOCATION, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_FixedLocationsComboStrings, wxCB_READONLY );
    itemBoxSizer20->Add(m_FixedLocationsCombo, 0, wxGROW|wxALL, 5);

#endif

    wxStaticText* itemStaticText24 = new wxStaticText( m_PanelFixed, wxID_STATIC, _("Enter or select the folder to catalog"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer20->Add(itemStaticText24, 0, wxALIGN_LEFT|wxLEFT|wxRIGHT, 5);

    wxBoxSizer* itemBoxSizer25 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer20->Add(itemBoxSizer25, 0, wxGROW, 5);

    m_FolderPathFixed = new wxTextCtrl( m_PanelFixed, ID_FIX_FOLDER_PATH, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer25->Add(m_FolderPathFixed, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton27 = new wxButton( m_PanelFixed, ID_FIXED_FOLDER_BROWSE, _("..."), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT );
    itemBoxSizer25->Add(itemButton27, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP|wxBOTTOM, 5);

    wxBoxSizer* itemBoxSizer28 = new wxBoxSizer(wxHORIZONTAL);
    m_MainSizer->Add(itemBoxSizer28, 0, wxGROW|wxTOP|wxBOTTOM, 5);

    m_HelpButton = new wxButton( itemDialog1, wxID_HELP, _("&Help"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer28->Add(m_HelpButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer28->Add(5, 5, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CatalogButton = new wxButton( itemDialog1, ID_BUTTON_CATALOG, _("Catalog"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer28->Add(m_CatalogButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CloseButton = new wxButton( itemDialog1, wxID_CANCEL, _("Close"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer28->Add(m_CloseButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_LastCatalogedCtrl = new wxStaticText( itemDialog1, ID_LAST_CATALOGED, _("Last cataloged"), wxDefaultPosition, wxDefaultSize, 0 );
    m_MainSizer->Add(m_LastCatalogedCtrl, 0, wxALIGN_LEFT|wxALL, 5);

    m_ProgressBarCtrl = new wxGauge( itemDialog1, ID_PROGRESS_BAR, 100, wxDefaultPosition, wxSize(-1, 20), wxGA_HORIZONTAL );
    m_ProgressBarCtrl->SetValue(0);
    m_MainSizer->Add(m_ProgressBarCtrl, 0, wxGROW|wxALL, 5);

////@end CDialogCatalogImages content construction
}


/*!
 * Should we show tooltips?
 */

bool CDialogCatalogImages::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CDialogCatalogImages::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CDialogCatalogImages bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CDialogCatalogImages bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CDialogCatalogImages::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CDialogCatalogImages icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CDialogCatalogImages icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_VOLUME_BROWSE
 */

void CDialogCatalogImages::OnVolumeBrowseClick( wxCommandEvent& event )
{
	wxDirDialog dlg( this, _("Select the volume to catalog"), m_VolumePath->GetValue(), wxDD_DEFAULT_STYLE|wxDD_DIR_MUST_EXIST );
	if( dlg.ShowModal() == wxID_OK ) {
		m_VolumePath->SetValue( dlg.GetPath() );
		m_FolderPathRemovable->SetValue( m_VolumePath->GetValue() );
	}
}

#if defined(__WXMSW__)

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GET_VOLUME_NAME
 */

void CDialogCatalogImages::OnGetVolumeNameClick( wxCommandEvent& event )
{
	wxString path = m_VolumePath->GetValue();
	if( !path.EndsWith(wxT("\\")) )
		path += wxT("\\");
	wxString vn = GetVolumeName( path );
	m_VolumeName->SetValue( vn );
	m_realVolumeName = vn;
}
#endif


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_FOLDER_BROWSE
 */

void CDialogCatalogImages::OnFolderBrowseClick( wxCommandEvent& event )
{
	wxDirDialog dlg( this, _("Select the path to catalog"), m_FolderPathRemovable->GetValue(), wxDD_DEFAULT_STYLE|wxDD_DIR_MUST_EXIST );
	if( dlg.ShowModal() == wxID_OK ) {
		wxString volumePath = m_VolumePath->GetValue();
		wxString folderPath = dlg.GetPath();
		if( folderPath.StartsWith(volumePath) ) {
			m_FolderPathRemovable->SetValue( dlg.GetPath() );
		}
		else {
			CUtils::MsgErr( _("The removable folder to catalog must be contained in the selected volume") );
		}
	}
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
 */

void CDialogCatalogImages::OnFixedFolderBrowseClick( wxCommandEvent& event )
{
	if( m_CurrentFixedLocationComboIndex < 0 ) {
		CUtils::MsgErr( _("You must select a fixed location") );
		return;
	}

	wxDirDialog dlg( this, _("Select the path to catalog"), m_FolderPathFixed->GetValue(), wxDD_DEFAULT_STYLE|wxDD_DIR_MUST_EXIST );
	if( dlg.ShowModal() == wxID_OK ) {
		CFixedLocations& fl = wxGetApp().GetFixedLocations();
		wxString fixedLocationPath = fl[m_CurrentFixedLocationComboIndex].GetPath();
		wxString folderPath = dlg.GetPath();
		if( folderPath.StartsWith(fixedLocationPath) ) {
			m_FolderPathFixed->SetValue( dlg.GetPath() );
		}
		else {
			CUtils::MsgErr( _("The fixed folder to catalog must be contained in the selected fixed location") );
		}
	}
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
 */

void CDialogCatalogImages::OnHelpClick( wxCommandEvent& WXUNUSED(event) )
{
	wxGetApp().GetHelpController()->DisplaySection( wxT("catalog_images.htm") );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_CATALOG
 */

void CDialogCatalogImages::OnButtonCatalogClick( wxCommandEvent& event )
{
	if( cataloging ) {
		cancelCatalogTask = true;
		return;
	}

	wxString removablePath, fixedPath;	// paths to catalog
	wxString volumeName, volumePath;	// name and path of the fixed volume
	wxString fixedLocationName, fixedLocationPath;			// name and path of the fixed location
	wxString catalogedFolder = wxEmptyString;	// stores the name of the currently cataloged folder to show it at the end
	
	CFixedLocations& fl = wxGetApp().GetFixedLocations();

	// check input
	if( !m_ChkHandleRemovable->IsChecked() && !m_ChkHandleFixed->IsChecked() ) {
		CUtils::MsgErr( _("You must choose to handle fixed volumes, removable volumes or both") );
		return;
	}
	if( m_ChkHandleRemovable->IsChecked() ) {
		volumeName = m_VolumeName->GetValue();
		if( volumeName.Trim().empty() ) {
			CUtils::MsgErr( _("The volume name is missing") );
			return;
		}
		volumePath = m_VolumePath->GetValue();
		if( volumePath.Trim().empty() ) {
			CUtils::MsgErr( _("The volume path is missing") );
			return;
		}
		removablePath = m_FolderPathRemovable->GetValue();
		if( removablePath.Trim().empty() ) {
			CUtils::MsgErr( _("The removable folder to catalog is missing") );
			return;
		}
		if( !removablePath.StartsWith(volumePath) ) {
			CUtils::MsgErr( _("The removable folder to catalog must be contained in the selected volume") );
			return;
		}
	}
	if( m_ChkHandleFixed->IsChecked() ) {
		if( m_CurrentFixedLocationComboIndex < 0 ) {
			CUtils::MsgErr( _("You must select a fixed location") );
			return;
		}
		fixedLocationName = fl[m_CurrentFixedLocationComboIndex].GetName();
		fixedLocationPath = fl[m_CurrentFixedLocationComboIndex].GetPath();
		fixedPath = m_FolderPathFixed->GetValue();
		if( fixedPath.Trim().empty() ) {
			CUtils::MsgErr( _("The fixed folder to catalog is missing") );
			return;
		}
		if( !fixedPath.StartsWith(fixedLocationPath) ) {
			CUtils::MsgErr( _("The fixed folder to catalog must be contained in the selected fixed location") );
		}
	}

	if( m_ChkHandleRemovable->IsChecked() && m_ChkHandleFixed->IsChecked() ) {
		// since we have both a fixed and removable path first check
		// that they contain the same files
		CCompareFolders cf( removablePath, fixedPath );
		cf.Compare();
		if( cf.FilesAreDifferent() ) {
			if( !ReportDifferentFiles(cf) ) {
				return;
			}
		}
		m_ProgressBarCtrl->SetRange( cf.GetFilesNumber() );
	}
	if( m_ChkHandleRemovable->IsChecked() && !m_ChkHandleFixed->IsChecked() ) {
		// only removable, we count the number of files
		CCompareFolders cf( removablePath, wxEmptyString );
		cf.Compare();
		m_ProgressBarCtrl->SetRange( cf.GetFilesNumber() );
	}
	if( !m_ChkHandleRemovable->IsChecked() && m_ChkHandleFixed->IsChecked() ) {
		// only fixed, we count the number of files
		CCompareFolders cf( fixedPath, wxEmptyString );
		cf.Compare();
		m_ProgressBarCtrl->SetRange( cf.GetFilesNumber() );
	}

	m_ProgressBarCtrl->SetValue( 0 );

	EnableDisableControls( false );
	
	// change the button text to "Stop"
	wxString orgCaption = m_CatalogButton->GetLabel();
	m_CatalogButton->SetLabel( _("Stop") );
	cataloging = true;
	cancelCatalogTask = false;	// will be set to true if the user clicks the button renamed to "Stop"

	CCatalogImagesFunctions cif( this, &cancelCatalogTask, m_ProgressBarCtrl );
	if( m_ChkHandleRemovable->IsChecked() ) {
		cif.SetRemovableData( volumeName, volumePath, removablePath );
		catalogedFolder = volumeName + wxT(" ") + removablePath;
	}
	if( m_ChkHandleFixed->IsChecked() ) {
		cif.SetFixedData( fixedLocationName, fixedLocationPath, fixedPath );
		if( catalogedFolder.empty() )
			catalogedFolder = fixedLocationName + wxT(" ") + fixedPath;
	}
	cif.Catalog();

	cataloging = false;
	cancelCatalogTask = false;
	m_CatalogButton->SetLabel( orgCaption );
	EnableDisableControls( true );

	if( m_ChkHandleRemovable->IsChecked() ) {
		m_VolumeName->SetValue( wxEmptyString );
		m_FolderPathRemovable->SetValue( m_VolumePath->GetValue() );
	}
	if( m_ChkHandleFixed->IsChecked() ) {
		m_FolderPathFixed->SetValue( fl[m_CurrentFixedLocationComboIndex].GetPath() );
	}

	// clear the progress bar
	m_ProgressBarCtrl->SetValue( 0 );

	// write the last cataloged folder as a reminder
	m_LastCatalogedCtrl->SetLabel( _("Last cataloged: ") + catalogedFolder );

	m_DatabaseChanged = true;

}

bool CDialogCatalogImages::ReportDifferentFiles( CCompareFolders cf ) {

	wxString msg = wxEmptyString;
	std::vector<wxString>::const_iterator vi;

	if( cf.GetFilesOnlyPath1().size() > 0 )
		msg += wxString(_("These files are only present in the removable location:")) + wxT("\n");
	for( vi = cf.GetFilesOnlyPath1().begin(); vi != cf.GetFilesOnlyPath1().end(); vi++ )
		msg += *vi + wxT("\n");

	if( cf.GetFilesOnlyPath2().size() > 0 )
		msg += wxString(_("These files are only present in the fixed location:")) + wxT("\n");
	for( vi = cf.GetFilesOnlyPath2().begin(); vi != cf.GetFilesOnlyPath2().end(); vi++ )
		msg += *vi + wxT("\n");
	
	if( cf.GetFilesDifferent().size() > 0 )
		msg += wxString(_("These files are different in size and/or date in the removable and fixed location:")) + wxT("\n");
	for( vi = cf.GetFilesDifferent().begin(); vi != cf.GetFilesDifferent().end(); vi++ )
		msg += *vi + wxT("\n");

	CLongTextQuestion dialog( this, ID_DIALOG_LONG_TEXT_QUESTION, _("Folders content warning") );
	dialog.SetUpperText( _("You have chosen to catalog both a fixed and a removable location,\nbut they do not contain exactly the same files.\nHere is the list of what is different.") );
	dialog.SetMidText( msg );
	dialog.SetLowerText( _("The program will catalog only the images contained in the removable location\nand it will ignore what is different in the fixed location.\nClick Yes to catalog the images, click No to cancel this task.\nDo you want to continue?") );
	dialog.SetHelpPage( wxT("ltq_catimg_folders_content.htm") );

	return (dialog.ShowModal() == wxID_YES);
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_PANEL_REMOVABLE
 */

void CDialogCatalogImages::OnPanelRemovableUpdate( wxUpdateUIEvent& event )
{
	m_PanelRemovable->Enable( m_ChkHandleRemovable->IsChecked() );

}


/*!
 * wxEVT_UPDATE_UI event handler for ID_PANEL_FIXED
 */

void CDialogCatalogImages::OnPanelFixedUpdate( wxUpdateUIEvent& event )
{
	m_PanelFixed->Enable( m_ChkHandleFixed->IsChecked() );
}


/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_FIXED_LOCATION
 */

void CDialogCatalogImages::OnChoiceFixedLocationSelected( wxCommandEvent& event )
{
	m_CurrentFixedLocationComboIndex = event.GetInt();
	CFixedLocations& fl = wxGetApp().GetFixedLocations();
	m_FolderPathFixed->SetValue( fl[m_CurrentFixedLocationComboIndex].GetPath() );
}

void CDialogCatalogImages::EnableDisableControls( bool enabled ) {

	m_HelpButton->Enable( enabled );
//	m_CatalogButton->Enable( enabled );
	m_CloseButton->Enable( enabled );

	if( !enabled ) {
		// store the previous state of the panels before disabling
		removablePanelEnabled = m_PanelRemovable->IsEnabled();
		fixedPanelEnabled = m_PanelFixed->IsEnabled();
		if( removablePanelEnabled ) m_PanelRemovable->Enable( false );
		if( fixedPanelEnabled ) m_PanelFixed->Enable( false );
	}
	else {
		if( removablePanelEnabled ) m_PanelRemovable->Enable( true );
		if( fixedPanelEnabled ) m_PanelFixed->Enable( true );
	}
}




/*!
 * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_HANDLE_FIXED
 */

void CDialogCatalogImages::OnHandleFixedClick( wxCommandEvent& event )
{
	if( m_ChkHandleFixed->IsChecked() ) {
		CFixedLocations& fl = wxGetApp().GetFixedLocations();
		if( fl.size() == 0 ) {
			// the user wants to catalog fixed locations but he has not defined any
			ShowWarningMissingFixedLocation();
		}
	}
}

void CDialogCatalogImages::ShowWarningMissingFixedLocation()
{
	wxString msg( wxT("You decided to catalog images in fixed locations but you have not defined any of them for the current catalog. You can define fixed locations with the \"Catalog/Fixed Location\" menu. You can find more information in the online help, looking at the \"Getting started\" chapter.") );
	CUtils::MsgInfo( msg );
}

