/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        choose_virtual_folder.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     07/03/2009 08:49:00
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "wx/imaglist.h"
////@end includes

#include "vvvpapp.h"
#include "choose_virtual_folder.h"
#include "treectrl_virtual.h"
#include "treectrl_virtual_itemdata.h"

#include <wx/clntdata.h>

////@begin XPM images
////@end XPM images


/*!
 * CDialogChooseVirtualFolder type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CDialogChooseVirtualFolder, wxDialog )


/*!
 * CDialogChooseVirtualFolder event table definition
 */

BEGIN_EVENT_TABLE( CDialogChooseVirtualFolder, wxDialog )

////@begin CDialogChooseVirtualFolder event table entries
#if defined(__WXMSW__) || defined(__WXMAC__)
    EVT_CHOICE( ID_CHOICE_RECENT, CDialogChooseVirtualFolder::OnChoiceRecentSelected )
#endif

#if defined(__WXGTK__)
    EVT_COMBOBOX( ID_COMBO_RECENT, CDialogChooseVirtualFolder::OnChoiceRecentSelected )
#endif

    EVT_BUTTON( wxID_OK, CDialogChooseVirtualFolder::OnOkClick )

////@end CDialogChooseVirtualFolder event table entries

END_EVENT_TABLE()


// class to handle wxArrayString(s) associated with combo items
class ASCliendData : public wxClientData
{
protected:

	wxArrayString as;

public:

	ASCliendData( wxArrayString& as )
	{
		wxClientData();
		this->as = as;
	}

	wxArrayString& GetArrayString() { return as; }
};


/*!
 * CDialogChooseVirtualFolder constructors
 */

CDialogChooseVirtualFolder::CDialogChooseVirtualFolder()
{
    Init();
}

CDialogChooseVirtualFolder::CDialogChooseVirtualFolder( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CDialogChooseVirtualFolder creator
 */

bool CDialogChooseVirtualFolder::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CDialogChooseVirtualFolder creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CDialogChooseVirtualFolder creation

	m_WindowPosition.SetWindow( this );
	m_WindowPosition.RestorePosition();

	return true;
}


/*!
 * CDialogChooseVirtualFolder destructor
 */

CDialogChooseVirtualFolder::~CDialogChooseVirtualFolder()
{
////@begin CDialogChooseVirtualFolder destruction
////@end CDialogChooseVirtualFolder destruction

	m_WindowPosition.SavePosition();
}


/*!
 * Member initialisation
 */

void CDialogChooseVirtualFolder::Init()
{
////@begin CDialogChooseVirtualFolder member initialisation
#if defined(__WXMSW__) || defined(__WXMAC__)
    m_RecentFoldersCtrl = NULL;
#endif
#if defined(__WXGTK__)
    m_RecentFoldersCtrl = NULL;
#endif
    m_TreeCtrlVirtual = NULL;
////@end CDialogChooseVirtualFolder member initialisation
	m_VirtualFolderID = -1;
}


/*!
 * Control creation for CDialogChooseVirtualFolder
 */

void CDialogChooseVirtualFolder::CreateControls()
{    
////@begin CDialogChooseVirtualFolder content construction
    CDialogChooseVirtualFolder* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxStaticText* itemStaticText3 = new wxStaticText( itemDialog1, wxID_STATIC, _("Please select the virtual folder that will receive the selected files and/or folders:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText3, 0, wxALIGN_LEFT|wxALL, 5);

#if defined(__WXMSW__) || defined(__WXMAC__)
    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemDialog1, wxID_STATIC, _("Recent:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(itemStaticText5, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP|wxBOTTOM, 5);

    wxArrayString m_RecentFoldersCtrlStrings;
    m_RecentFoldersCtrl = new wxChoice( itemDialog1, ID_CHOICE_RECENT, wxDefaultPosition, wxDefaultSize, m_RecentFoldersCtrlStrings, 0 );
    itemBoxSizer4->Add(m_RecentFoldersCtrl, 1, wxALIGN_CENTER_VERTICAL|wxLEFT|wxTOP|wxBOTTOM, 5);

#endif

#if defined(__WXGTK__)
    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer7, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText8 = new wxStaticText( itemDialog1, wxID_STATIC, _("Recent:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer7->Add(itemStaticText8, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP|wxBOTTOM, 5);

    wxArrayString m_RecentFoldersCtrlStrings;
    m_RecentFoldersCtrl = new wxComboBox( itemDialog1, ID_COMBO_RECENT, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_RecentFoldersCtrlStrings, wxCB_READONLY );
    itemBoxSizer7->Add(m_RecentFoldersCtrl, 1, wxALIGN_CENTER_VERTICAL|wxLEFT|wxTOP|wxBOTTOM, 5);

#endif

    m_TreeCtrlVirtual = new CTreeCtrlVirtual( itemDialog1, ID_TREECTRL, wxDefaultPosition, wxSize(-1, 300), wxTR_HAS_BUTTONS |wxTR_HIDE_ROOT|wxTR_SINGLE|wxTR_DEFAULT_STYLE );
    itemBoxSizer2->Add(m_TreeCtrlVirtual, 1, wxGROW|wxALL, 5);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer11 = new wxStdDialogButtonSizer;

    itemBoxSizer2->Add(itemStdDialogButtonSizer11, 0, wxGROW|wxALL, 10);
    wxButton* itemButton12 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer11->AddButton(itemButton12);

    wxButton* itemButton13 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer11->AddButton(itemButton13);

    itemStdDialogButtonSizer11->Realize();

////@end CDialogChooseVirtualFolder content construction

	m_TreeCtrlVirtual->LoadData();

	// fill the combo box with the last used virtual folders
	CAppSettings& as = wxGetApp().GetAppSettings();
	wxString sep = wxFileName::GetPathSeparator();
	for( size_t k = 0; k < as.m_AddVirtualFoldersLastUsed.size(); k++ ) {
		wxArrayString& ars = as.m_AddVirtualFoldersLastUsed[k];
		wxString folderName = wxEmptyString;
		for( int i = ars.GetCount() - 1; i >= 0; i-- ) {
			if( !folderName.empty() ) folderName += sep;
			folderName += ars[i];
		}
		m_RecentFoldersCtrl->Append( folderName, new ASCliendData(ars) );
	}
}


/*!
 * Should we show tooltips?
 */

bool CDialogChooseVirtualFolder::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CDialogChooseVirtualFolder::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CDialogChooseVirtualFolder bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CDialogChooseVirtualFolder bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CDialogChooseVirtualFolder::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CDialogChooseVirtualFolder icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CDialogChooseVirtualFolder icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CDialogChooseVirtualFolder::OnOkClick( wxCommandEvent& WXUNUSED(event) )
{
	wxTreeItemId itemID = m_TreeCtrlVirtual->GetSelection();
	if( itemID.IsOk() ) {
	    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) m_TreeCtrlVirtual->GetItemData(itemID);
		if( itemData != NULL )
			m_VirtualFolderID = itemData->GetPathID();
		else
			m_VirtualFolderID = -1;
	}
	else {
		m_VirtualFolderID = -1;
	}

	// add the selected folder to the list of last used folders
	if( m_VirtualFolderID != -1 ) {
		CAppSettings& as = wxGetApp().GetAppSettings();
		wxArrayString ars;
		GetCurrentPathArray( ars );
		// first remove the folder from the list, if present
		for( size_t k = 0; k < as.m_AddVirtualFoldersLastUsed.size(); k++ ) {
			if( as.m_AddVirtualFoldersLastUsed[k] == ars )
				as.m_AddVirtualFoldersLastUsed.erase( as.m_AddVirtualFoldersLastUsed.begin() + k );
		}
		// now create a new vector with the correct content
		std::vector<wxArrayString> tmp;
		int nAdded = 0;
		tmp.push_back( ars );
		nAdded++;
		for( size_t k = 0; k < as.m_AddVirtualFoldersLastUsed.size(); k++ ) {
			tmp.push_back( as.m_AddVirtualFoldersLastUsed[k] );
			nAdded++;
			if( nAdded == as.m_MaxAddVirtualFoldersLastUsed )
				break;
		}
		// swap vectors
		as.m_AddVirtualFoldersLastUsed = tmp;
	}

	EndModal( wxID_OK );
}

void CDialogChooseVirtualFolder::GetCurrentPathArray( wxArrayString& as ) {

	as.Empty();
	wxTreeItemId itemID = m_TreeCtrlVirtual->GetSelection();
	
	if( !itemID.IsOk() ) return;		// empty array

	while( itemID != m_TreeCtrlVirtual->GetRootItem() ) {
		as.Add( m_TreeCtrlVirtual->GetItemText(itemID) );
		itemID = m_TreeCtrlVirtual->GetItemParent( itemID );
	}
}

void CDialogChooseVirtualFolder::SetCurrentPathArray( wxArrayString& as ) {
	int i;

	if( as.GetCount() == 0 ) return;

	wxTreeItemId fatherID = m_TreeCtrlVirtual->GetRootItem();
	i = (int) as.GetCount() - 1;
	while( i >= 0 ) {
		// selects the right child of the current item
		wxTreeItemIdValue cookie;
		wxTreeItemId childID = m_TreeCtrlVirtual->GetFirstChild( fatherID, cookie );
		while( childID.IsOk() ) {
			wxString aa = m_TreeCtrlVirtual->GetItemText(childID);
			wxString bb = as[i];
			if( m_TreeCtrlVirtual->GetItemText(childID) == as[i] ) {
				// found! select this item
				m_TreeCtrlVirtual->SelectItem( childID );
				fatherID = childID;
				wxSafeYield();
				break;
			}

			childID = m_TreeCtrlVirtual->GetNextChild( fatherID, cookie );
			if( !childID.IsOk() )
				return;		// we did not find the folder
		}

		i--;
	}
}



/*!
 * wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_RECENT
 */

void CDialogChooseVirtualFolder::OnChoiceRecentSelected( wxCommandEvent& event )
{
	int i = m_RecentFoldersCtrl->GetSelection();
	if( i == wxNOT_FOUND ) return;

	ASCliendData *pt = dynamic_cast<ASCliendData *>( m_RecentFoldersCtrl->GetClientObject(i) );
	wxASSERT( pt != NULL );
	wxArrayString& as = pt->GetArrayString();
	SetCurrentPathArray( as );
}

