/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2011, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "db_file_list.h"
#include "vvvpapp.h"
#include "sqlstrings.h"

#include "fsdb.h"

using namespace FSDB;


CDbFileList::CDbFileList( FSDB::Database db )
{
	m_db = db;
}

CDbFileList::~CDbFileList(void)
{
}

void CDbFileList::CreateFileListFolder( const int volumeID, const int folderID, const wxString &folderPath )
{
	AddSingleFolderToList( volumeID, folderID, folderPath );
}


void CDbFileList::AddSingleFolderToList( const int volumeID, const int pathID, const wxString &path )
{
	// add the files to the list
	wxString sep = wxString(wxFileName::GetPathSeparator());
	FSDB_CREATE_RS_READONLY_FWDONLY( m_db, rsFiles );
	std::string sql = CSqlStrings::SelectFilesInAPath( pathID );
	rsFiles.Open( sql );
	while( !rsFiles.IsEOF() ) {
		CDbFileListData fd;
		fd.path = path + sep + CUtils::DBstd2wx(rsFiles["FILE_NAME"]);
		fd.SHA1 = CUtils::DBstd2wx(rsFiles["SHA1_HASH"]);
		m_FileList.push_back( fd );

		rsFiles.MoveNext();
	}
	rsFiles.Close();

	// recurse subfolders
	std::vector<int> subPathIDs;
	std::vector<wxString> subPathNames;
	FSDB_CREATE_RS_READONLY_FWDONLY( m_db, rsFolders );
	sql = CSqlStrings::SelectPathsList( volumeID, pathID );
	rsFolders.Open( sql );
	while( !rsFolders.IsEOF() ) {
		subPathIDs.push_back( rsFolders["PATH_ID"] );
		subPathNames.push_back( CUtils::DBstd2wx(rsFolders["PATH_NAME"]) );
		rsFolders.MoveNext();
	}
	rsFolders.Close();

	for( size_t k = 0; k < subPathIDs.size(); k++ ) {
		AddSingleFolderToList( volumeID, subPathIDs[k], path + sep + subPathNames[k] );
	}

}

