/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _FIXED_LOCATIONS_H_
#define _FIXED_LOCATIONS_H_

#include <vector>

#include <wx/string.h>


// this class stores the data about a singlke fixed location
class CSingleFixedLocation {

	friend class CFixedLocations;

private:

	wxString name;				// location name
	wxString path;				// physical path
	wxString description;		// optional description

public:

	const wxString& GetName() { return name; }
	const wxString& GetPath() { return path; }
	const wxString& GetDescription() { return description; }

};

/*
this class handles the list of fixed locations defined for a UUID value
*/
class CFixedLocations
{
public:
	CFixedLocations(void);
	~CFixedLocations(void);

	enum Error {
		eNoError,
		eDuplicateName,		/* the new name is already present in the list */
		eDuplicatePath,		/* the new path is already present in the list */
		ePathContained,		/* the new path is contained in another path present in the list */
		ePathContains,		/* the new path contains another path present in the list */
		eMissingName,		/* the location name is missing*/
		eMissingPath		/* the path is missing */
	};

	// remove everything from the array
	void Clear();

	// return an element of the array
	CSingleFixedLocation& operator[]( int index ) { return *dataArray[index]; }

	// return the number of elements in the array
	int size() { return (int) dataArray.size(); }

	// append a new fixed location at the end of the container
	Error Append( wxString name, wxString path, wxString description );

	// insert the data at a given index, overwriting the old data
	Error Edit( wxString name, wxString path, wxString description, int index );

	// remove the row at the given index from the container
	void Remove( int index );

	// load locations from the user profile. It will use the current UUID as a key to select data
	void Load();

	// save locations to the user profile. It will use the current UUID as a key to select data
	void Save();

	// set the UUID to be used as a key. It will cause a Load() of the stored locations
	void SetUUID( wxString sUUID );

	// return the path of a fixed location given its name, or an empty string if the name is not found
	wxString GetPathByName( wxString locationName );

private:
	
	wxString FixedLocationUUID;		// the UUID for which we are handling the locations

	std::vector<CSingleFixedLocation*> dataArray;

	// check if the data do not conflict with the data already contained in the list
	// if this is a new row index = -1 (impossible value)
	// if this is an edited line index is the index of the data to be changed
	Error CheckData( wxString name, wxString path, wxString description, int index );

};

#endif
