/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        fixed_locations_dialog.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     22/10/2008 12:33:18
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "wx/imaglist.h"
////@end includes

#include "fixed_locations_dialog.h"
#include "vvvpapp.h"
#include "utils.h"

////@begin XPM images
////@end XPM images


/*!
 * CFixedLocationsDialog type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CFixedLocationsDialog, wxDialog )


/*!
 * CFixedLocationsDialog event table definition
 */

BEGIN_EVENT_TABLE( CFixedLocationsDialog, wxDialog )

////@begin CFixedLocationsDialog event table entries
    EVT_SIZE( CFixedLocationsDialog::OnSize )

    EVT_BUTTON( ID_BUTTON_ADD, CFixedLocationsDialog::OnButtonAddClick )

    EVT_BUTTON( ID_BUTTON_EDIT, CFixedLocationsDialog::OnButtonEditClick )

    EVT_BUTTON( ID_BUTTON_DELETE, CFixedLocationsDialog::OnButtonDeleteClick )

    EVT_BUTTON( wxID_HELP, CFixedLocationsDialog::OnHelpClick )

    EVT_BUTTON( ID_BUTTON_BROWSE, CFixedLocationsDialog::OnButtonBrowseClick )

    EVT_BUTTON( ID_DLG_OK, CFixedLocationsDialog::OnDlgOkClick )

    EVT_BUTTON( ID_DLG_CANCEL, CFixedLocationsDialog::OnDlgCancelClick )

////@end CFixedLocationsDialog event table entries

END_EVENT_TABLE()


/*!
 * CFixedLocationsDialog constructors
 */

CFixedLocationsDialog::CFixedLocationsDialog()
{
    Init();
}

CFixedLocationsDialog::CFixedLocationsDialog( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CFixedLocationsDialog creator
 */

bool CFixedLocationsDialog::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CFixedLocationsDialog creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CFixedLocationsDialog creation

	FillListControl();
	HideShowLowerPart( false );
	m_EditStatus = esNoEdit;

	m_WindowPosition.SetWindow( this );
	m_WindowPosition.RestorePosition();

	ResizeLCColumns();

	return true;
}


/*!
 * CFixedLocationsDialog destructor
 */

CFixedLocationsDialog::~CFixedLocationsDialog()
{
////@begin CFixedLocationsDialog destruction
////@end CFixedLocationsDialog destruction
	m_WindowPosition.SavePosition();
}


/*!
 * Member initialisation
 */

void CFixedLocationsDialog::Init()
{
////@begin CFixedLocationsDialog member initialisation
    m_ListCtrl = NULL;
    m_AddButton = NULL;
    m_EditButton = NULL;
    m_DeleteButton = NULL;
    m_CloseButton = NULL;
    m_LocationLabel = NULL;
    m_LocationCtrl = NULL;
    m_PhysicalPathLabel = NULL;
    m_PhysicalPathCtrl = NULL;
    m_BrowseCtrl = NULL;
    m_DescriptionLabel = NULL;
    m_DescriptionCtrl = NULL;
    m_OkButton = NULL;
    m_CancelButton = NULL;
////@end CFixedLocationsDialog member initialisation
}


/*!
 * Control creation for CFixedLocationsDialog
 */

void CFixedLocationsDialog::CreateControls()
{    
////@begin CFixedLocationsDialog content construction
    CFixedLocationsDialog* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    m_ListCtrl = new wxListCtrl( itemDialog1, ID_LISTCTRL, wxDefaultPosition, wxSize(100, 100), wxLC_REPORT|wxLC_NO_SORT_HEADER|wxLC_SINGLE_SEL|wxSUNKEN_BORDER );
    itemBoxSizer2->Add(m_ListCtrl, 1, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer4, 0, wxGROW|wxALL, 5);

    m_AddButton = new wxButton( itemDialog1, ID_BUTTON_ADD, _("Add"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(m_AddButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_EditButton = new wxButton( itemDialog1, ID_BUTTON_EDIT, _("Edit"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(m_EditButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_DeleteButton = new wxButton( itemDialog1, ID_BUTTON_DELETE, _("&Delete"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(m_DeleteButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton( itemDialog1, wxID_HELP, _("&Help"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(itemButton8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer4->Add(5, 5, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CloseButton = new wxButton( itemDialog1, wxID_CLOSE, _("&Close"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(m_CloseButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticLine* itemStaticLine11 = new wxStaticLine( itemDialog1, wxID_STATIC, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );
    itemBoxSizer2->Add(itemStaticLine11, 0, wxGROW|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer12 = new wxFlexGridSizer(3, 2, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer12, 0, wxGROW|wxALL, 5);

    m_LocationLabel = new wxStaticText( itemDialog1, wxID_STATIC, _("Location name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer12->Add(m_LocationLabel, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_LocationCtrl = new wxTextCtrl( itemDialog1, ID_NAME, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer12->Add(m_LocationCtrl, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_PhysicalPathLabel = new wxStaticText( itemDialog1, wxID_STATIC, _("Physical path"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer12->Add(m_PhysicalPathLabel, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer16 = new wxBoxSizer(wxHORIZONTAL);
    itemFlexGridSizer12->Add(itemBoxSizer16, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP|wxBOTTOM, 5);

    m_PhysicalPathCtrl = new wxTextCtrl( itemDialog1, ID_PATH, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer16->Add(m_PhysicalPathCtrl, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_BrowseCtrl = new wxButton( itemDialog1, ID_BUTTON_BROWSE, _("..."), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT );
    itemBoxSizer16->Add(m_BrowseCtrl, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_DescriptionLabel = new wxStaticText( itemDialog1, wxID_STATIC, _("Description"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer12->Add(m_DescriptionLabel, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_DescriptionCtrl = new wxTextCtrl( itemDialog1, ID_DESCRIPTION, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer12->Add(m_DescriptionCtrl, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemFlexGridSizer12->AddGrowableCol(1);

    wxBoxSizer* itemBoxSizer21 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer21, 0, wxALIGN_RIGHT|wxALL, 5);

    m_OkButton = new wxButton( itemDialog1, ID_DLG_OK, _("OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer21->Add(m_OkButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_CancelButton = new wxButton( itemDialog1, ID_DLG_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer21->Add(m_CancelButton, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CFixedLocationsDialog content construction

	CreateLCHeaders();

//	SetEscapeId( wxID_CLOSE );
	SetAffirmativeId( wxID_CLOSE );
}


/*!
 * Should we show tooltips?
 */

bool CFixedLocationsDialog::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CFixedLocationsDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CFixedLocationsDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CFixedLocationsDialog bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CFixedLocationsDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CFixedLocationsDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CFixedLocationsDialog icon retrieval
}

void CFixedLocationsDialog::HideShowLowerPart( bool visible ) {

	if( !visible ) {
		m_LocationCtrl->SetValue( wxEmptyString );
		m_PhysicalPathCtrl->SetValue( wxEmptyString );
		m_DescriptionCtrl->SetValue( wxEmptyString );
	}
	m_LocationLabel->Enable( visible );
	m_LocationCtrl->Enable( visible );
	m_PhysicalPathLabel->Enable( visible );
	m_PhysicalPathCtrl->Enable( visible );
	m_BrowseCtrl->Enable( visible );
	m_DescriptionLabel->Enable( visible );
	m_DescriptionCtrl->Enable( visible );
	m_OkButton->Enable( visible );
	m_CancelButton->Enable( visible );

	m_AddButton->Enable( !visible );
	m_EditButton->Enable( !visible );
	m_DeleteButton->Enable( !visible );
	m_CloseButton->Enable( !visible );
	m_ListCtrl->Enable( !visible );

}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ADD
 */

void CFixedLocationsDialog::OnButtonAddClick( wxCommandEvent& event )
{
	wxASSERT( m_EditStatus == esNoEdit );

	HideShowLowerPart( true );
	m_LocationCtrl->SetFocus();
	m_EditStatus = esAppend;
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EDIT
 */

void CFixedLocationsDialog::OnButtonEditClick( wxCommandEvent& event )
{
	wxASSERT( m_EditStatus == esNoEdit );

	long i = GetLCSelectedItemIndex();
	if( i < 0 ) return;

	HideShowLowerPart( true );
	m_LocationCtrl->SetFocus();
	m_EditStatus = esEdit;

	CFixedLocations& fl = wxGetApp().GetFixedLocations();
	m_LocationCtrl->SetValue( fl[i].GetName() );
	m_PhysicalPathCtrl->SetValue( fl[i].GetPath() );
	m_DescriptionCtrl->SetValue( fl[i].GetDescription() );

}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DLG_OK
 */

void CFixedLocationsDialog::OnDlgOkClick( wxCommandEvent& event )
{
	int index = -1;

	wxASSERT( m_EditStatus == esEdit || m_EditStatus == esAppend );

	CFixedLocations& fl = wxGetApp().GetFixedLocations();

	// check data
	wxString name = m_LocationCtrl->GetValue();
	wxString path = m_PhysicalPathCtrl->GetValue();
	wxString description = m_DescriptionCtrl->GetValue();
	index = GetLCSelectedItemIndex();
	CFixedLocations::Error err;
	if( m_EditStatus == esAppend ) {
		err = fl.Append( name, path, description );
	}
	else {
		err = fl.Edit( name, path, description, index );
	}
	if( err != CFixedLocations::eNoError ) {
		wxString msg;
		switch( err ) {
			case CFixedLocations::eMissingName:
				msg = _("The location name is missing");
				break;
			case CFixedLocations::eMissingPath:
				msg = _("The physical path is missing");
				break;
			case CFixedLocations::eDuplicateName:
				msg = _("The location name is already present in the list");
				break;
			case CFixedLocations::eDuplicatePath:
				msg = _("The physical path is already present in the list");
				break;
			case CFixedLocations::ePathContained:
				msg = _("The physical path would be contained in one of the paths in the list");
				break;
			case CFixedLocations::ePathContains:
				msg = _("The physical path would contain one of the paths in the list");
				break;
		}
		CUtils::MsgErr( msg );
		return;
	}

	// write data to the list control
	FillListControl();
	fl.Save();

	HideShowLowerPart( false );
	m_EditStatus = esNoEdit;
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DLG_CANCEL
 */

void CFixedLocationsDialog::OnDlgCancelClick( wxCommandEvent& event )
{
	wxASSERT( m_EditStatus == esEdit || m_EditStatus == esAppend );

	HideShowLowerPart( false );
	m_EditStatus = esNoEdit;
}

void CFixedLocationsDialog::CreateLCHeaders() {

	wxListItem itemCol;
	int i = 0;

	itemCol.SetText( _("Name") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	m_ListCtrl->InsertColumn( i++, itemCol );

	itemCol.SetText( _("Physical Path") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	m_ListCtrl->InsertColumn( i++, itemCol );

	itemCol.SetText( _("Description") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	m_ListCtrl->InsertColumn( i++, itemCol );
}


/*!
 * wxEVT_SIZE event handler for ID_CFIXEDLOCATIONSDIALOG
 */

void CFixedLocationsDialog::OnSize( wxSizeEvent& event )
{
	event.Skip();
	ResizeLCColumns();
}

void CFixedLocationsDialog::ResizeLCColumns() {
	int x, y;

    if( m_ListCtrl == NULL ) return;

	m_ListCtrl->GetClientSize( &x, &y );
	m_ListCtrl->SetColumnWidth( 0, x/3 );
	m_ListCtrl->SetColumnWidth( 1, x/3 );
	m_ListCtrl->SetColumnWidth( 2, x - (x/3)*2 - 2 );
}

void CFixedLocationsDialog::FillListControl() {

	m_ListCtrl->DeleteAllItems();
	CFixedLocations& fl = wxGetApp().GetFixedLocations();
	for( int k = 0; k < fl.size(); k++ ) {
    	int i = m_ListCtrl->InsertItem( 0, "" );

	    // this code is used to set the item's mask
	    wxListItem item;
	    item.SetId( i );
	    m_ListCtrl->GetItem( item );
	    item.SetMask( wxLIST_MASK_STATE|wxLIST_MASK_TEXT|wxLIST_MASK_DATA );
	    m_ListCtrl->SetItem( item );

		int col = 0;
        int flIndex = fl.size() - k - 1;	// items are inserted at the top of the list
		m_ListCtrl->SetItem( i, col, fl[flIndex].GetName() );
		col++;
		m_ListCtrl->SetItem( i, col, fl[flIndex].GetPath() );
		col++;
		m_ListCtrl->SetItem( i, col, fl[flIndex].GetDescription() );
		col++;
	}
}

long CFixedLocationsDialog::GetLCSelectedItemIndex() {

	int index = -1;
	index = m_ListCtrl->GetNextItem( index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
	return index;
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_BROWSE
 */

void CFixedLocationsDialog::OnButtonBrowseClick( wxCommandEvent& event )
{
	wxDirDialog dlg( this, _("Select the path to the fixed location"), m_PhysicalPathCtrl->GetValue(), wxDD_DEFAULT_STYLE|wxDD_DIR_MUST_EXIST );
	if( dlg.ShowModal() == wxID_OK )
		m_PhysicalPathCtrl->SetValue( dlg.GetPath() );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
 */

void CFixedLocationsDialog::OnHelpClick( wxCommandEvent& WXUNUSED(event) )
{
	wxGetApp().GetHelpController()->DisplaySection( wxT("fixed_locations.htm") );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_DELETE
 */

void CFixedLocationsDialog::OnButtonDeleteClick( wxCommandEvent& event )
{
	wxASSERT( m_EditStatus == esNoEdit );

	long i = GetLCSelectedItemIndex();
	if( i < 0 ) return;

	wxString msg = wxT("This command will delete the fixed location from the list shown in this window, but it will not delete any file or folder from your computer.");
	msg += wxT("\n\n");
	msg += wxT("Do you really want to delete the selected fixed location?");

	if( !CUtils::MsgAskNo(msg) ) return;

	CFixedLocations& fl = wxGetApp().GetFixedLocations();
	fl.Remove( i );
	FillListControl();
	fl.Save();
}

