/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        fixed_locations_dialog.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     22/10/2008 12:33:18
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _FIXED_LOCATIONS_DIALOG_H_
#define _FIXED_LOCATIONS_DIALOG_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/listctrl.h"
#include "wx/statline.h"
////@end includes

#include "window_position.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxListCtrl;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CFIXEDLOCATIONSDIALOG 10036
#define ID_LISTCTRL 10038
#define ID_BUTTON_ADD 10037
#define ID_BUTTON_EDIT 10039
#define ID_BUTTON_DELETE 10135
#define ID_NAME 10040
#define ID_PATH 10041
#define ID_BUTTON_BROWSE 10046
#define ID_DESCRIPTION 10042
#define ID_DLG_OK 10044
#define ID_DLG_CANCEL 10045
#define SYMBOL_CFIXEDLOCATIONSDIALOG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CFIXEDLOCATIONSDIALOG_TITLE _("Fixed Locations")
#define SYMBOL_CFIXEDLOCATIONSDIALOG_IDNAME ID_CFIXEDLOCATIONSDIALOG
#define SYMBOL_CFIXEDLOCATIONSDIALOG_SIZE wxSize(400, 300)
#define SYMBOL_CFIXEDLOCATIONSDIALOG_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CFixedLocationsDialog class declaration
 */

class CFixedLocationsDialog: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CFixedLocationsDialog )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CFixedLocationsDialog();
    CFixedLocationsDialog( wxWindow* parent, wxWindowID id = SYMBOL_CFIXEDLOCATIONSDIALOG_IDNAME, const wxString& caption = SYMBOL_CFIXEDLOCATIONSDIALOG_TITLE, const wxPoint& pos = SYMBOL_CFIXEDLOCATIONSDIALOG_POSITION, const wxSize& size = SYMBOL_CFIXEDLOCATIONSDIALOG_SIZE, long style = SYMBOL_CFIXEDLOCATIONSDIALOG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CFIXEDLOCATIONSDIALOG_IDNAME, const wxString& caption = SYMBOL_CFIXEDLOCATIONSDIALOG_TITLE, const wxPoint& pos = SYMBOL_CFIXEDLOCATIONSDIALOG_POSITION, const wxSize& size = SYMBOL_CFIXEDLOCATIONSDIALOG_SIZE, long style = SYMBOL_CFIXEDLOCATIONSDIALOG_STYLE );

    /// Destructor
    ~CFixedLocationsDialog();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CFixedLocationsDialog event handler declarations

    /// wxEVT_SIZE event handler for ID_CFIXEDLOCATIONSDIALOG
    void OnSize( wxSizeEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ADD
    void OnButtonAddClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_EDIT
    void OnButtonEditClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_DELETE
    void OnButtonDeleteClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
    void OnHelpClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_BROWSE
    void OnButtonBrowseClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DLG_OK
    void OnDlgOkClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DLG_CANCEL
    void OnDlgCancelClick( wxCommandEvent& event );

////@end CFixedLocationsDialog event handler declarations

////@begin CFixedLocationsDialog member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CFixedLocationsDialog member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CFixedLocationsDialog member variables
    wxListCtrl* m_ListCtrl;
    wxButton* m_AddButton;
    wxButton* m_EditButton;
    wxButton* m_DeleteButton;
    wxButton* m_CloseButton;
    wxStaticText* m_LocationLabel;
    wxTextCtrl* m_LocationCtrl;
    wxStaticText* m_PhysicalPathLabel;
    wxTextCtrl* m_PhysicalPathCtrl;
    wxButton* m_BrowseCtrl;
    wxStaticText* m_DescriptionLabel;
    wxTextCtrl* m_DescriptionCtrl;
    wxButton* m_OkButton;
    wxButton* m_CancelButton;
////@end CFixedLocationsDialog member variables

private:

	CWindowPosition m_WindowPosition;

	// hide (visible=false) or show (visible=true) the controls in the lower part of the dialog
	void HideShowLowerPart( bool visible );

	// create the list control's headers
	void CreateLCHeaders();

	// resize the list control's columns to fit the control's width
	void ResizeLCColumns();

	// fill the list control with the current fixed locations
	void FillListControl();

	// current status of the lower part of the window
	enum EditStatus {
		esNoEdit,	/* controls are not enabled */
		esAppend,	/* the user has pressed the Append button */
		esEdit		/* the user has pressed the Edit button */
	};
	EditStatus m_EditStatus;

	// return the 0-based id of the selected item, -1 if no item is selected
	long GetLCSelectedItemIndex();

};

#endif
    // _FIXED_LOCATIONS_DIALOG_H_
