/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2011, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
This class encapsulates the usage of the LCMS library
*/

#ifndef _FS_LCMS_H_
#define _FS_LCMS_H_

#include "memory_chunk.h"
#include "lcms2.h"
#include <wx/image.h>

class CColorProfile {

public:

	enum ReturnValue { 
		rvOk,
		rvFileMissing,	// the profile file is missing, switched to sRGB
		rvFileInvalid	// the profile file is invalid, switched to sRGB
	};

	CColorProfile();
	~CColorProfile();

	// assign an sRGB profile for output
	void Assign_sRGBOutputProfile();

	// assign the given profile for output
	ReturnValue AssignFileOutputProfile( const wxString& profileFile );

	// transform an image from the given profile (as a memory chunk) to the output one
	// NOTE: this function will change the image data: you probably will not want to save the converted image
	void TransformImage( wxImage &img, const CMemoryChunk &colorProfile );

	// transform an image from Adobe RGB using a built-in profile Adobe RGB
	void TransformImageFromAdobeRGB( wxImage &img );

private:

	// transform an image from the given input profile to the output profile
	void TransformImage( wxImage &img, const cmsHPROFILE inputProfile );

	// create a clone of the Adobe RGB profile. Code is taken from the lcms test program.
	cmsHPROFILE Create_AdobeRGB();

	// output profile used by the transform
	cmsHPROFILE m_OutputProfile;

	// true if output profile is sRGB: used as a flag
	bool m_OutputProfileIs_sRGB;

	// an in-memory copy of a clone of the Adobe RGB profile
	cmsHPROFILE m_AdobeRGBProfile;

};

#endif
