/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        image_information_dialog.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     17/04/2009 10:18:58
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _IMAGE_INFORMATION_DIALOG_H_
#define _IMAGE_INFORMATION_DIALOG_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/listctrl.h"
////@end includes

#include "window_position.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxListCtrl;
////@end forward declarations

class CFileData;
class CFileDataArray;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CIMAGEINFORMATIONDIALOG 10075
#define ID_LISTCTRL1 10076
#define ID_TEXTCTRL 10077
#define SYMBOL_CIMAGEINFORMATIONDIALOG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CIMAGEINFORMATIONDIALOG_TITLE _("Image information")
#define SYMBOL_CIMAGEINFORMATIONDIALOG_IDNAME ID_CIMAGEINFORMATIONDIALOG
#define SYMBOL_CIMAGEINFORMATIONDIALOG_SIZE wxSize(100, 100)
#define SYMBOL_CIMAGEINFORMATIONDIALOG_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CImageInformationDialog class declaration
 */

class CImageInformationDialog: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CImageInformationDialog )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CImageInformationDialog();
    CImageInformationDialog( wxWindow* parent, wxWindowID id = SYMBOL_CIMAGEINFORMATIONDIALOG_IDNAME, const wxString& caption = SYMBOL_CIMAGEINFORMATIONDIALOG_TITLE, const wxPoint& pos = SYMBOL_CIMAGEINFORMATIONDIALOG_POSITION, const wxSize& size = SYMBOL_CIMAGEINFORMATIONDIALOG_SIZE, long style = SYMBOL_CIMAGEINFORMATIONDIALOG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CIMAGEINFORMATIONDIALOG_IDNAME, const wxString& caption = SYMBOL_CIMAGEINFORMATIONDIALOG_TITLE, const wxPoint& pos = SYMBOL_CIMAGEINFORMATIONDIALOG_POSITION, const wxSize& size = SYMBOL_CIMAGEINFORMATIONDIALOG_SIZE, long style = SYMBOL_CIMAGEINFORMATIONDIALOG_STYLE );

    /// Destructor
    ~CImageInformationDialog();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CImageInformationDialog event handler declarations

    /// wxEVT_SIZE event handler for ID_CIMAGEINFORMATIONDIALOG
    void OnSize( wxSizeEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
    void OnOkClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
    void OnHelpClick( wxCommandEvent& event );

////@end CImageInformationDialog event handler declarations

////@begin CImageInformationDialog member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CImageInformationDialog member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CImageInformationDialog member variables
    wxListCtrl* m_ListCtrl;
    wxTextCtrl* m_TextCtrl;
////@end CImageInformationDialog member variables

	// set the data to show in the window
	void SetFileData( CFileData* fileData );

	// get the description
	wxString GetDescription( void );

private:

	// create the list control headers
	void CreateLCHeaders();

	// pointer to the data to show
	CFileData* m_fileData;

	// fill the dialog with image data
	void ShowFileData();

	// add a row to the list control
	// return the index of the inserted row
	int AddLCRow( wxString col0, wxString col1 );

	// set the columns size to a reasonable value
	void ResizeLCColumns();

	CWindowPosition m_WindowPosition;

};

#endif
    // _IMAGE_INFORMATION_DIALOG_H_
