/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    aint with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        image_viewer_frame.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     10/08/2009 14:06:28
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _IMAGE_VIEWER_FRAME_H_
#define _IMAGE_VIEWER_FRAME_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/frame.h"
#include "wx/toolbar.h"
#include "wx/statusbr.h"
////@end includes

#include "window_position.h"
#include "fs_imageviewer.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
class FS_ImageViewer;
class wxStatusBar;
////@end forward declarations

class CFileData;
class CFileDataArray;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CIMAGEVIEWERFRAME 10091
#define ID_IMGVIEWER_TOOLBAR 10092
#define ID_IMGV_VIEW_FULLSCREEN 10007
#define ID_IMGV_VIEW_FIRST 10003
#define ID_IMGV_VIEW_PREVIOUS 10004
#define ID_IMGV_VIEW_NEXT 10005
#define ID_IMGV_VIEW_LAST 10006
#define ID_IMGV_VIEW_FIT 10000
#define ID_IMGV_VIEW_FIT_H 10001
#define ID_IMGV_VIEW_FIT_V 10002
#define ID_IMGV_VIEW_REAL_SIZE 10093
#define ID_IMGV_VIEW_ZOOM_IN 10094
#define ID_IMGV_VIEW_ZOOM_OUT 10095
#define ID_IMGV_EDIT_IMAGE_INFORMATION 10096
#define ID_PANEL_VIEWER 10097
#define ID_IMAGE_VIEWER_CTRL 10089
#define ID_IMGVIEWER_STATUSBAR 10087
#define SYMBOL_CIMAGEVIEWERFRAME_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxMAXIMIZE_BOX|wxCLOSE_BOX
#define SYMBOL_CIMAGEVIEWERFRAME_TITLE _("Image Viewer")
#define SYMBOL_CIMAGEVIEWERFRAME_IDNAME ID_CIMAGEVIEWERFRAME
#define SYMBOL_CIMAGEVIEWERFRAME_SIZE wxSize(400, 300)
#define SYMBOL_CIMAGEVIEWERFRAME_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CImageViewerFrame class declaration
 */

class CImageViewerFrame: public wxFrame
{    
    DECLARE_CLASS( CImageViewerFrame )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CImageViewerFrame();
    CImageViewerFrame( wxWindow* parent, wxWindowID id = SYMBOL_CIMAGEVIEWERFRAME_IDNAME, const wxString& caption = SYMBOL_CIMAGEVIEWERFRAME_TITLE, const wxPoint& pos = SYMBOL_CIMAGEVIEWERFRAME_POSITION, const wxSize& size = SYMBOL_CIMAGEVIEWERFRAME_SIZE, long style = SYMBOL_CIMAGEVIEWERFRAME_STYLE );

    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CIMAGEVIEWERFRAME_IDNAME, const wxString& caption = SYMBOL_CIMAGEVIEWERFRAME_TITLE, const wxPoint& pos = SYMBOL_CIMAGEVIEWERFRAME_POSITION, const wxSize& size = SYMBOL_CIMAGEVIEWERFRAME_SIZE, long style = SYMBOL_CIMAGEVIEWERFRAME_STYLE );

    /// Destructor
    ~CImageViewerFrame();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CImageViewerFrame event handler declarations

    /// wxEVT_CLOSE_WINDOW event handler for ID_CIMAGEVIEWERFRAME
    void OnCloseWindow( wxCloseEvent& event );

    /// wxEVT_ERASE_BACKGROUND event handler for ID_CIMAGEVIEWERFRAME
    void OnEraseBackground( wxEraseEvent& event );

    /// wxEVT_CHAR event handler for ID_CIMAGEVIEWERFRAME
    void OnChar( wxKeyEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_EDIT_IMAGE_INFORMATION
    void OnImgvEditImageInformationClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FULLSCREEN
    void OnImgvViewFullscreenClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIT
    void OnImgvViewFitClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIT_H
    void OnImgvViewFitHClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIT_V
    void OnImgvViewFitVClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_REAL_SIZE
    void OnImgvViewRealSizeClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_REAL_SIZE
    void OnImgvViewRealSizeUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_ZOOM_IN
    void OnImgvViewZoomInClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_IN
    void OnImgvViewZoomInUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_ZOOM_OUT
    void OnImgvViewZoomOutClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_OUT
    void OnImgvViewZoomOutUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIRST
    void OnImgvViewFirstClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_FIRST
    void OnImgvViewFirstUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_PREVIOUS
    void OnImgvViewPreviousClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_PREVIOUS
    void OnImgvViewPreviousUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_NEXT
    void OnImgvViewNextClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_NEXT
    void OnImgvViewNextUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_LAST
    void OnImgvViewLastClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_LAST
    void OnImgvViewLastUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_HELP
    void OnHelpClick( wxCommandEvent& event );

////@end CImageViewerFrame event handler declarations

	// handler for the "zoom changed" event from the viewer control
	void OnViewerZoomChanged( fsImageViewerEvent& event );

////@begin CImageViewerFrame member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CImageViewerFrame member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

	// set the filedata array that contains the data of the images currently shown
	void SetFiledataArray( CFileDataArray *fda );
	// set the index to the image to show in the filedata array
	void SetFiledataArrayIndex( int index );

	// show the image pointed by the current index in the filedata array
	void ShowCurrentImage();

	// true if the window was maximized bafore hiding it
	bool WasMaximized() { return m_WasMaximized; }

	// setup color management from the current options
	// it is called each time the user options are changed
	void SetupColorManagement();

////@begin CImageViewerFrame member variables
    wxToolBar* m_ToolbarCtrl;
    wxPanel* m_ViewerPanel;
    wxBoxSizer* m_ViewerSizer;
    FS_ImageViewer* m_ImageViewerCtrl;
    wxStatusBar* m_StatusBar;
////@end CImageViewerFrame member variables

private:

	// values for the Status bar elements
	enum StatusBarElements {
		sbeMenuDescription = 0,
		sbeDimensions,
		sbePosition,
		sbeZoom,
		sbeDateTime,
		sbeDescription
	};

	// move to the next image
	void MoveNext();
	// move to the previous image
	void MovePrevious();

	// update the status bar
	void UpdateStatusBar();

	// create a toolbar for the Mac version ( bitmaps are enlarged )
	void CreateMacToolbar();

	// go full screen
	void StartFullScreen();
	// end full screen
	void EndFullScreen();

	// data of the images to show
	CFileDataArray *m_FileDataArray;
	// index of the currently shown image
	int m_FileDataIndex;

	CWindowPosition m_WindowPosition;

	// remember if the window was maximized before hiding
	bool m_WasMaximized;

	// to apply color profiles
	CColorProfile m_ColorProfilesHandler;

    // the image viewer control has a border that makes it look better
    // the border is bad in full screen so we create another control without border and make it visible when viewing at full screen
    // this pointer always points to the control that is not currently visible: it will be switched when needed
    FS_ImageViewer* m_ImageViewerInvisibleCtrl;
};

#endif
    // _IMAGE_VIEWER_FRAME_H_
