/////////////////////////////////////////////////////////////////////////////
// Name:        long_text_question.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     03/01/2009 15:29:57
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

/*
this is a dialog that can be used to show a complex question regarding a list of items to the user.
It is divided in three parts:

- The top part is a static text that shows introductory info. It is set by SetUpperText()
- The middle part is a read/only text box that can show a high number of lines. It is possibile to select the
lines and copy them to the clipboard. This text is set by SetMidText()
- The lower part is a static text that shows some final text, usually a question. It is set by SetLowerText()
- The help button is available if the help page to show has been set by SetHelpPage()

The dialog shows yes and no buttons and it returns wxID_YES if the user clicked the yes button

*/

#ifndef _LONG_TEXT_QUESTION_H_
#define _LONG_TEXT_QUESTION_H_


/*!
 * Includes
 */

////@begin includes
////@end includes

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_DIALOG_LONG_TEXT_QUESTION 10058
#define ID_CENTRAL_TEXT 10059
#define SYMBOL_CLONGTEXTQUESTION_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CLONGTEXTQUESTION_TITLE _("Complex question")
#define SYMBOL_CLONGTEXTQUESTION_IDNAME ID_DIALOG_LONG_TEXT_QUESTION
#define SYMBOL_CLONGTEXTQUESTION_SIZE wxDefaultSize
#define SYMBOL_CLONGTEXTQUESTION_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CLongTextQuestion class declaration
 */

class CLongTextQuestion: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CLongTextQuestion )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CLongTextQuestion();
    CLongTextQuestion( wxWindow* parent, wxWindowID id = SYMBOL_CLONGTEXTQUESTION_IDNAME, const wxString& caption = SYMBOL_CLONGTEXTQUESTION_TITLE, const wxPoint& pos = SYMBOL_CLONGTEXTQUESTION_POSITION, const wxSize& size = SYMBOL_CLONGTEXTQUESTION_SIZE, long style = SYMBOL_CLONGTEXTQUESTION_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CLONGTEXTQUESTION_IDNAME, const wxString& caption = SYMBOL_CLONGTEXTQUESTION_TITLE, const wxPoint& pos = SYMBOL_CLONGTEXTQUESTION_POSITION, const wxSize& size = SYMBOL_CLONGTEXTQUESTION_SIZE, long style = SYMBOL_CLONGTEXTQUESTION_STYLE );

    /// Destructor
    ~CLongTextQuestion();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CLongTextQuestion event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
    void OnHelpClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for wxID_HELP
    void OnHelpUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_YES
    void OnYesClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_NO
    void OnNoClick( wxCommandEvent& event );

////@end CLongTextQuestion event handler declarations

////@begin CLongTextQuestion member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CLongTextQuestion member function declarations

	void SetUpperText( wxString msg ) { m_UpperTextCtrl->SetLabel( msg ); AdaptSize(); }
	void SetMidText( wxString msg ) { m_MidTextCtrl->SetValue( msg ); }
	void SetLowerText( wxString msg ) { m_LowerTextCtrl->SetLabel( msg ); AdaptSize(); }
	void SetHelpPage( wxString hp ) { m_HelpPage = hp; }

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CLongTextQuestion member variables
    wxBoxSizer* m_MainSizer;
    wxStaticText* m_UpperTextCtrl;
    wxTextCtrl* m_MidTextCtrl;
    wxStaticText* m_LowerTextCtrl;
////@end CLongTextQuestion member variables

private:

	// name of the help page to show if the user presses the help button
	wxString m_HelpPage;

	// resize the dialog to fit to control that have changed size
	void AdaptSize();
};

#endif
    // _LONG_TEXT_QUESTION_H_
