/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        mainframe.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     30/07/2008 21:53:32
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _MAINFRAME_H_
#define _MAINFRAME_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/frame.h"
#include "wx/toolbar.h"
#include "wx/splitter.h"
////@end includes

#include "app_settings.h"
#include "rightpanelist.h"
#include "vvvp_thumbnailctrl.h"
#include "filedata_array.h"
#include "left_right_panes_events.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxMenu;
class wxSplitterWindow;
class CTreeCtrlPhysical;
class CRightPaneList;
class CStatusBarWithGauge;
////@end forward declarations

class CTreeCtrlVirtual;
class CSearchPanel;
class CTimelinePanel;
class CImageViewerFrame;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_MAINFRAME 10000
#define ID_TOOLBAR 10009
#define ID_CATALOG_VOLUME 10001
#define ID_EDIT_OBJECT_INFORMATION 10074
#define ID_VIEW_FIXED_LOCATIONS 10060
#define ID_VIEW_REMOVABLE_LOCATIONS 10061
#define ID_VIEW_VIRTUAL_LOCATIONS 10065
#define ID_VIEW_TIMELINE_VIEW 10171
#define ID_VIEW_SEARCH_VIEW 10081
#define ID_VIEW_LARGE_THUMBNAILS 10034
#define ID_VIEW_MEDIUM_THUMBNAILS 10033
#define ID_VIEW_SMALL_THUMBNAILS 10032
#define ID_VIEW_NO_THUMBNAILS 10031
#define ID_FILE_CHECK_INTEGRITY 10168
#define ID_FILE_BACKUP 10131
#define ID_FILE_RESTORE 10132
#define ID_COPY_FILES 10082
#define ID_ADD_VIRTUAL_FOLDER 10068
#define ID_EDIT_ROTATE 10028
#define ID_EDIT_ROTATE_RIGHT 10062
#define ID_EDIT_ROTATE_UPSIDE_DOWN 10063
#define ID_EDIT_ROTATE_LEFT 10064
#define ID_EDIT_RENAME 10119
#define ID_EDIT_DELETE 10026
#define ID_EDIT_THUMBNAILS 10120
#define ID_EDIT_THUMBNAILS_RESIZE 10124
#define ID_EDIT_THUMBNAILS_DELETE 10121
#define ID_NEW_VIRTUAL_ROOT_FOLDER 10069
#define ID_NEW_VIRTUAL_SUBFOLDER 10070
#define ID_EDIT_EP 10148
#define ID_EDIT_EP0 10149
#define ID_EDIT_EP1 10150
#define ID_EDIT_EP2 10151
#define ID_EDIT_EP3 10152
#define ID_EDIT_EP4 10153
#define ID_EDIT_EP5 10154
#define ID_EDIT_EP6 10155
#define ID_EDIT_EP7 10156
#define ID_EDIT_EP8 10157
#define ID_EDIT_EP9 10158
#define ID_FIXED_LOCATIONS 10043
#define ID_VIEW_TOOLBAR 10079
#define ID_VIEW_STATUS_BAR 10080
#define ID_TOOLS_SHOW_IMAGE 10090
#define ID_SPLITTERWINDOW 10027
#define SYMBOL_CMAINFRAME_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxMINIMIZE_BOX|wxMAXIMIZE_BOX|wxCLOSE_BOX
#define SYMBOL_CMAINFRAME_TITLE _("vvvP")
#define SYMBOL_CMAINFRAME_IDNAME ID_MAINFRAME
#define SYMBOL_CMAINFRAME_SIZE wxSize(600, 400)
#define SYMBOL_CMAINFRAME_POSITION wxDefaultPosition
////@end control identifiers

#define ID_THUMBNAILS_CTRL 10002

/*!
 * CMainFrame class declaration
 */

class CMainFrame: public wxFrame
{    
    DECLARE_CLASS( CMainFrame )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CMainFrame();
    CMainFrame( wxWindow* parent, wxWindowID id = SYMBOL_CMAINFRAME_IDNAME, const wxString& caption = SYMBOL_CMAINFRAME_TITLE, const wxPoint& pos = SYMBOL_CMAINFRAME_POSITION, const wxSize& size = SYMBOL_CMAINFRAME_SIZE, long style = SYMBOL_CMAINFRAME_STYLE );

    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CMAINFRAME_IDNAME, const wxString& caption = SYMBOL_CMAINFRAME_TITLE, const wxPoint& pos = SYMBOL_CMAINFRAME_POSITION, const wxSize& size = SYMBOL_CMAINFRAME_SIZE, long style = SYMBOL_CMAINFRAME_STYLE );

    /// Destructor
    ~CMainFrame();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CMainFrame event handler declarations

    /// wxEVT_CLOSE_WINDOW event handler for ID_MAINFRAME
    void OnCloseWindow( wxCloseEvent& event );

    /// wxEVT_IDLE event handler for ID_MAINFRAME
    void OnIdle( wxIdleEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_NEW
    void OnNewClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_OPEN
    void OnOpenClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_FILE_CHECK_INTEGRITY
    void OnFileCheckIntegrityClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_FILE_CHECK_INTEGRITY
    void OnFileCheckIntegrityUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_FILE_BACKUP
    void OnFileBackupClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_FILE_BACKUP
    void OnFileBackupUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_FILE_RESTORE
    void OnFileRestoreClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_COPY_FILES
    void OnCopyFilesClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_COPY_FILES
    void OnCopyFilesUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_ADD_VIRTUAL_FOLDER
    void OnAddVirtualFolderClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_ADD_VIRTUAL_FOLDER
    void OnAddVirtualFolderUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_SELECTALL
    void OnSelectallClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for wxID_SELECTALL
    void OnSelectallUpdate( wxUpdateUIEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE
    void OnEditRotateUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_ROTATE_RIGHT
    void OnEditRotateRightClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE_RIGHT
    void OnEditRotateRightUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_ROTATE_UPSIDE_DOWN
    void OnEditRotateUpsideDownClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE_UPSIDE_DOWN
    void OnEditRotateUpsideDownUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_ROTATE_LEFT
    void OnEditRotateLeftClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE_LEFT
    void OnEditRotateLeftUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_RENAME
    void OnEditRenameClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_RENAME
    void OnEditRenameUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_DELETE
    void OnEditDeleteClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_DELETE
    void OnEditDeleteUpdate( wxUpdateUIEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_THUMBNAILS
    void OnEditThumbnailsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_THUMBNAILS_RESIZE
    void OnEditThumbnailsResizeClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_THUMBNAILS_RESIZE
    void OnEditThumbnailsResizeUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_THUMBNAILS_DELETE
    void OnEditThumbnailsDeleteClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_THUMBNAILS_DELETE
    void OnEditThumbnailsDeleteUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_OBJECT_INFORMATION
    void OnEditObjectInformationClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_EDIT_OBJECT_INFORMATION
    void OnEditObjectInformationUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_NEW_VIRTUAL_ROOT_FOLDER
    void OnNewVirtualRootFolderClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_NEW_VIRTUAL_ROOT_FOLDER
    void OnNewVirtualRootFolderUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_NEW_VIRTUAL_SUBFOLDER
    void OnNewVirtualSubfolderClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_NEW_VIRTUAL_SUBFOLDER
    void OnNewVirtualSubfolderUpdate( wxUpdateUIEvent& event );

#if defined(__WXMAC__) || defined(__WXGTK__)
    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_PREFERENCES
    void OnPreferencesClick( wxCommandEvent& event );

#endif
    /// wxEVT_UPDATE_UI event handler for ID_EDIT_EP
    void OnEditEpUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_EP0
    void OnEditExternalProgramsClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_CATALOG_VOLUME
    void OnCatalogVolumeClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_CATALOG_VOLUME
    void OnCatalogVolumeUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_FIXED_LOCATIONS
    void OnFixedLocationsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_FIXED_LOCATIONS
    void OnFixedLocationsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_FIXED_LOCATIONS
    void OnViewFixedLocationsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_FIXED_LOCATIONS
    void OnViewFixedLocationsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_REMOVABLE_LOCATIONS
    void OnViewRemovableLocationsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_REMOVABLE_LOCATIONS
    void OnViewRemovableLocationsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_VIRTUAL_LOCATIONS
    void OnViewVirtualLocationsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_VIRTUAL_LOCATIONS
    void OnViewVirtualLocationsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_TIMELINE_VIEW
    void OnViewTimelineViewClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_TIMELINE_VIEW
    void OnViewTimelineViewUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_SEARCH_VIEW
    void OnViewSearchViewClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_SEARCH_VIEW
    void OnViewSearchViewUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_NO_THUMBNAILS
    void OnViewNoThumbnailsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_NO_THUMBNAILS
    void OnViewNoThumbnailsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_SMALL_THUMBNAILS
    void OnViewSmallThumbnailsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_SMALL_THUMBNAILS
    void OnViewSmallThumbnailsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_MEDIUM_THUMBNAILS
    void OnViewMediumThumbnailsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_MEDIUM_THUMBNAILS
    void OnViewMediumThumbnailsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_LARGE_THUMBNAILS
    void OnViewLargeThumbnailsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_VIEW_LARGE_THUMBNAILS
    void OnViewLargeThumbnailsUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_TOOLBAR
    void OnViewToolbarClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_STATUS_BAR
    void OnViewStatusBarClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for ID_TOOLS_SHOW_IMAGE
    void OnToolsShowImageClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_TOOLS_SHOW_IMAGE
    void OnToolsShowImageUpdate( wxUpdateUIEvent& event );

#if defined(__WXMSW__)
    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_PREFERENCES
    void OnPreferencesClick( wxCommandEvent& event );

#endif
    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_HELP_CONTENTS
    void OnHelpContentsClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_MENU_SELECTED event handler for wxID_ABOUT
    void OnAboutClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_SPLITTER_DOUBLECLICKED event handler for ID_SPLITTERWINDOW
    void OnSplitterwindowSashDClick( wxSplitterEvent& event );

////@end CMainFrame event handler declarations

	// handler for double click on a thumbnail
	void OnThumbnailCtrlLeftDClick( wxThumbnailEvent& event );

	// handler for double click on a listview element
	void OnListviewItemActivated( wxListEvent& event );

	// handler for listview item selected
	void OnListviewItemSelected( wxListEvent& event );

	// handler for thumbnail item selected
	void OnThumbnailCtrlSelected( wxThumbnailEvent& event );

	// handler for listview item deselected
	void OnListviewItemDeselected( wxListEvent& event );

	// handler for thumbnail item deselected
	void OnThumbnailCtrlDeselected( wxThumbnailEvent& event );

	// handler for left pane focused
	void OnLeftPaneFocused( fsLeftPaneFocusedEvent& event );

	// handler for right pane focused
	void OnRightPaneFocused( fsRightPaneFocusedEvent& event );

////@begin CMainFrame member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CMainFrame member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

	// called when the user chooses a file from the MRU list
	void OnMRUFile( wxCommandEvent& event );

////@begin CMainFrame member variables
    wxToolBar* m_ToolbarCtrl;
    wxMenu* m_FileMenu;
    wxMenu* m_EditMenu;
    wxMenu* m_ViewMenu;
    wxSplitterWindow* m_SplitterWindow;
    CTreeCtrlPhysical* m_TreeCtrlRemovable;
    CRightPaneList* m_RightPaneListRemovable;
    CStatusBarWithGauge* m_StatusBar;
////@end CMainFrame member variables

	// controls created by hand (not by DialogBlocks) -----------
	
	// thumbnails for the removable view
	vvvPThumbnailCtrl* m_ThumbnailsCtrlRemovable;
	
	//   fixed view:
	// left pane tree control
	CTreeCtrlPhysical* m_TreeCtrlFixed;
	// right pane list control
    CRightPaneList* m_RightPaneListFixed;
	// right pane thumbnails
	vvvPThumbnailCtrl* m_ThumbnailsCtrlFixed;

	//   virtual view:
	// left pane tree control
	CTreeCtrlVirtual* m_TreeCtrlVirtual;
	// right pane list control
    CRightPaneList* m_RightPaneListVirtual;
	// right pane thumbnails
	vvvPThumbnailCtrl* m_ThumbnailsCtrlVirtual;

	//   search view
	// search panel
    CSearchPanel* m_SearchPanel;
	// right pane list control
    CRightPaneList* m_RightPaneListSearch;
	// right pane thumbnails
	vvvPThumbnailCtrl* m_ThumbnailsCtrlSearch;

    // timeline view
    // timeline panel
    CTimelinePanel* m_TimelinePanel;
	// right pane list control
    CRightPaneList* m_RightPaneListTimeline;
	// right pane thumbnails
	vvvPThumbnailCtrl* m_ThumbnailsCtrlTimeline;

	/// public methods

	/// show the contents of a path in the physical view
	void ShowFolderDataPhysical( long pathID );

	/// show the contents of a path in the virtual view
	void ShowFolderDataVirtual( long pathID );

	/// show the result of a search in the search pane
	void ShowSearchData( std::string sql );

	/// these methods are used for showing the results of more than one sql query
	/// store the image data found using the sql statement in the search data array
	/// if append == true data is appended to existing data
	/// if append == false data is cleared before storing
	/// if isVirtual == true we are looking in virtual folders, otherwise we are looking in physical folders
	void StoreSearchData( std::string sql, bool isVirtual, bool append );
	/// show the search data contents
	/// if sort == true sort the search data before showing it
	void ShowSearchData( bool sort );
	/// clear the search data list, called before recursively searching folder
	void ClearSearchData();

    // show data for the timeline view: all the images with a given date
    void ShowTimelineData( const wxDateTime &dt );

	/// return the current view
	CAppSettings::CurrentView GetCurrentView() { return m_CurrentView; }

	// return a pointer to the current thumbnail control (removable, fixed...)
	vvvPThumbnailCtrl* GetCurrentThumbnailCtrl();

	// return a pointer to the current listview control
	CRightPaneList* GetCurrentRightPaneListCtrl();

	// append the menus containing the edit/external programs entries to the specified menu
	void AppendExternalProgramsMenus( wxMenu *menu, CExternalPrograms& ep );

private:

	// values for the Status bar elements
	enum StatusBarElements {
		sbeMenuDescription = 0,
		sbeObjectsNumber,
		sbeObjectsSize,
		sbeObjectDescription
	};

	/// update the content of the status bar: it does not use the first element because it is used by the menu/toolbar
	void UpdateStatusBar();

	/// get a pointer to the window currently shown in the right pane,
	/// depending on the current user selection ( fixed/removable... and thumbnails/no thumbnails)
	wxWindow* GetCorrectWindowInRightPane();

	/// get a pointer to the window currently shown in the left pane,
	/// depending on the current user selection ( fixed/removable... )
	wxWindow* GetCorrectWindowInLeftPane();

	/// show the correct window in the rigt pane
	void ShowCorrectWindowInRightPane();

	/// show a window in the right pane
	void ShowWindowInRightPane( wxWindow* newWindow );

	/// set the "no thumbnails" view
	void SetViewNoThumbnails();
	/// set the small thumbnails view
	void SetViewSmallThumbnails();
	/// set the medium thumbnails view
	void SetViewMediumThumbnails();
	/// set the large thumbnails view
	void SetViewLargeThumbnails();

	/// show a window in the left pane
	void ShowWindowInLeftPane( wxWindow* newWindow );

	// set the "fixed locations" view
	void SetViewFixedLocations();
	// set the "removable locations" view
	void SetViewRemovableLocations();
	// set the virtual view
	void SetViewVirtual();
	// set the search view
	void SetViewSearch();
	// set the timeline view
	void SetViewTimeline();

	/// Open a database.
	/// Returns true if OK, returns false if not opened.
	bool OpenDatabase( wxString fileName, int expectedVersion );

	/// clear all the right pane controls, removing any data
	void ClearAllRightPanes();

	/// clear the the windows that are currently used in the right pane (thumbnails and listview)
	void ClearCurrentRightPanes();

	/// return true if the "Edit/Delete" menu must be enabled
	bool IsEditDeleteEnabled();

	/// delete the selected physical volume
	void DeleteSelectedPhysicalVolume();

	// delete the physical file(s) currently selected
	void DeleteSelectedPhysicalFiles();

	// delete the physical folder currently selected
	void DeleteSelectedPhysicalFolder();

	// delete the virtual file(s) currently selected
	void DeleteSelectedVirtualFiles();

	// delete the virtual folder currently selected
	void DeleteSelectedVirtualFolder();

	/// return true if we must enable the menus about image rotation
	bool EnableRotateMenus();

	/// return true if we must enable the menus about image thumbnails
	bool EnableEditThumbnailsMenus();

	/// return true if we must enable the menus about external programs
	bool EnableEditExternalProgramsMenus();

	/// return true if the right pane has the focus
	bool RightPaneHasFocus();

	/// return the filedata array of the current view (removable, fixed...)
	CFileDataArray *GetCurrentFileDataArray();

	/// show the currently selected image in the image viewer window
	void ShowSelectedImage();

	// create a new virtual folder from user input
	// FatherID is the primary key of the father of this folder in the database: it can be -1 (it means NULL) for root folders
	// windowTitle is the title of the window that will ask for the new folder's name
	void CreateNewVirtualFolder( int fatherID, wxString windowTitle );

	// restore a database from the given backup file. Shows the given caption in the save dialog box.
	// databaseFile is the name of the database to create
	// returns false if the database has not been restored, or true if it has been restored
	bool RestoreDatabase( wxString caption, wxString backupName, wxString& databaseFile );

	// select in the listview the same items selected in the thumbnails control
	// eventIndex is the index of the item that caused the selection event
	void CopySelectionFromThumbnailsToListview( int eventIndex );

	// select in the thumbnails control the same items selected in the listview
	// eventIndex is the index of the item that caused the selection event
	void CopySelectionFromListviewToThumbnails( int eventIndex );

	// create a toolbar for the Mac version ( bitmaps are enlarged )
	void CreateMacToolbar();

	// create the menu containing the edit/external programs entries
	void CreateExternalProgramsMenu();

	/// Contains the data of the currently selected removable folder
	CFileDataArray m_DataArrayRemovable;
	/// Contains the data of the currently selected fixed folder
	CFileDataArray m_DataArrayFixed;
	/// Contains the data of the currently selected virtual folder
	CFileDataArray m_DataArrayVirtual;
	/// Contains the data of the timeline view
	CFileDataArray m_DataArrayTimeline;
	/// Contains the data of the search view
	CFileDataArray m_DataArraySearch;

	/// the current thumbnails size
	CAppSettings::ThumbnailSize m_CurrentThumbnailSize;

	/// the current view
	CAppSettings::CurrentView m_CurrentView;

	// used to save the selected folder in the "Add to virtual folder" dialog box
	wxArrayString m_SelectedVirtualFolderForAdd;

	// frame used to show images
	CImageViewerFrame *m_ImgViewerFrame;

	// flag: true if we are handling a selection event in the thumbnails or listview control
	// used to avoid double handling when selecting items by software
	bool m_HandlingSelectionEvent;

	// number of rows to read from the recordset in a single idle event handling when loading thumbnails
	static const int DB_ROWS_READ_CHUNK = 10;

	// pointer to a recordset used to load data for the fixed view
	// data are loaded a bit at a time in the idle event handler
	FSDB::RecordsetPt rsPtFixed;
	// true if we are currently loading data in the fixed view
	bool m_LoadingFixedView;
	// number of rows to load from the database (only meaningful if m_LoadingFixedView == true)
	int m_NumRowsToLoadFixed;

	// pointer to a recordset used to load data for the removable view
	FSDB::RecordsetPt rsPtRemovable;
	// true if we are currently loading data in the removable view
	bool m_LoadingRemovableView;
	// number of rows to load from the database (only meaningful if m_LoadingRemovableView == true)
	int m_NumRowsToLoadRemovable;

	// pointer to a recordset used to load data for the virtual view
	FSDB::RecordsetPt rsPtVirtual;
	// true if we are currently loading data in the virtual view
	bool m_LoadingVirtualView;
	// number of rows to load from the database (only meaningful if m_LoadingVirtualView == true)
	int m_NumRowsToLoadVirtual;

	// pointer to a recordset used to load data for the timeline view
	FSDB::RecordsetPt rsPtTimeline;
	// true if we are currently loading data in the timeline view
	bool m_LoadingTimelineView;
	// number of rows to load from the database (only meaningful if m_LoadingTimelineView == true)
	int m_NumRowsToLoadTimeline;

	// true if the right pane has focus, false if the left pane has focus
	// changes only when a pane gets focus, using events sent from the panes
	bool m_RightPaneHasFocus;

};

#endif
    // _MAINFRAME_H_
