/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        rightpanelist.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     22/09/2008 11:54:01
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "wx/imaglist.h"
////@end includes

#include "rightpanelist.h"
#include "utils.h"
#include "vvvpapp.h"
#include "app_settings.h"
#include "image_information_dialog.h"
#include "object_description_dialog.h"
#include "sqlstrings.h"
#include "left_right_panes_events.h"

////@begin XPM images
////@end XPM images


/*!
 * CRightPaneList type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CRightPaneList, wxListCtrl )


/*!
 * CRightPaneList event table definition
 */

BEGIN_EVENT_TABLE( CRightPaneList, wxListCtrl )

////@begin CRightPaneList event table entries
    EVT_CONTEXT_MENU( CRightPaneList::OnContextMenu )
    EVT_SET_FOCUS( CRightPaneList::OnSetFocus )

////@end CRightPaneList event table entries

END_EVENT_TABLE()


/*!
 * CRightPaneList constructors
 */

CRightPaneList::CRightPaneList()
{
    Init();
}

CRightPaneList::CRightPaneList(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxValidator& validator)
{
    Init();
    Create(parent, id, pos, size, style, validator);
}


/*!
 * CRightPaneList creator
 */

bool CRightPaneList::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxValidator& validator)
{
////@begin CRightPaneList creation
    wxListCtrl::Create(parent, id, pos, size, style, validator);
    CreateControls();
////@end CRightPaneList creation
	CreateHeaders();
    return true;
}


/*!
 * CRightPaneList destructor
 */

CRightPaneList::~CRightPaneList()
{
////@begin CRightPaneList destruction
////@end CRightPaneList destruction

	StoreColumnsWidth();
}


/*!
 * Member initialisation
 */

void CRightPaneList::Init()
{
////@begin CRightPaneList member initialisation
////@end CRightPaneList member initialisation
	m_fileDataArray = NULL;
	descriptionColumn = -1;
	numFilesTot = 0;
	sizeFilesTot = 0;
}


/*!
 * Control creation for CRightPaneList
 */

void CRightPaneList::CreateControls()
{    
////@begin CRightPaneList content construction
////@end CRightPaneList content construction
}

void CRightPaneList::AddRowToListControl( CFileData& fd, long idx )
{
	numFilesTot++;
	sizeFilesTot += fd.GetSize();

	int i = InsertItem( idx, fd.GetName() );
	
	// this code is used to set the item's mask
	wxListItem item;
	item.SetId( i );
	GetItem( item );
	item.SetMask( wxLIST_MASK_STATE|wxLIST_MASK_TEXT|wxLIST_MASK_DATA );
	SetItem( item );

	SetItemData( i, (long) &fd );	// store the address of the CFileData item
	int col = 0;
	SetItem( i, col, fd.GetName() );
	col++;
	SetItem( i, col, CUtils::HumanReadableFileSize( fd.GetSize() ) );
	col++;
	if( fd.GetImageDateTime().IsValid() )
		SetItem( i, col, fd.GetImageDateTime().FormatDate() + wxT(" ") + fd.GetImageDateTime().FormatTime() );
	col++;
	SetItem( i, col, CUtils::NewLine2Space(fd.GetDescription()) );
	descriptionColumn = col;	// save this value for future use
	col++;

	CMetadata md = fd.GetMetadata();
	wxDateTime dt = md.GetImageTimeStamp();
	if( dt.IsValid() ) {
		SetItem( i, col, dt.FormatDate() + wxT(" ") + dt.FormatTime() );
		col++;
	}
	SetItem( i, col, md.GetCameraMake() );
	col++;
	SetItem( i, col, md.GetCameraModel() );
	col++;
	SetItem( i, col, md.GetSoftware() );
	col++;
	SetItem( i, col, md.GetExposureTime() );
	col++;
	SetItem( i, col, md.GetAperture() );
	col++;
	SetItem( i, col, md.GetMaxAperture() );
	col++;
	SetItem( i, col, md.GetExposureBias() );
	col++;
	SetItem( i, col, md.GetFlash() );
	col++;
	SetItem( i, col, md.GetFocalLength() );
	col++;
	SetItem( i, col, md.GetFocalLength35mm() );
	col++;
	SetItem( i, col, md.GetISOSpeed() );
	col++;
	SetItem( i, col, md.GetExposureMode() );
	col++;
	SetItem( i, col, md.GetMeteringMode() );
	col++;
	SetItem( i, col, md.GetImageQuality() );
	col++;
	SetItem( i, col, md.GetWhiteBalance() );
	col++;
}

void CRightPaneList::ShowContainerData( CFileDataArray &fileDataArray ) {
//	bool wasVisible;

	PrepareForLoading();
	
//	wasVisible = IsShown();
//	if( wasVisible ) Hide();	// to speed up things

	LoadNewData( fileDataArray );

//	if( wasVisible ) Show();
}

void CRightPaneList::PrepareForLoading()
{
	numFilesTot = 0;
	sizeFilesTot = 0;

	DeleteAllItems();
}

void CRightPaneList::LoadNewData( CFileDataArray &fileDataArray )
{
	int nCurrentItems = GetItemCount();
	m_fileDataArray = &fileDataArray;
	for( int k = nCurrentItems; k < fileDataArray.size(); k++ ) {
		AddRowToListControl( fileDataArray[k], k );
	}
}
void CRightPaneList::CreateHeaders() {
	ClearAll();

	// add the columns to the list control
	wxListItem itemCol;
	int i = 0;

	itemCol.SetText( _("Name") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Size") );
	itemCol.SetAlign( wxLIST_FORMAT_RIGHT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Image date") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Description") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Image timestamp") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Make") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Model") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Software") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Exp. time") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Aperture") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Max aperture") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Exp. bias") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Flash") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Focal length") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("35mm eq.") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("ISO") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Exp. mode") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Metering mode") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("Image quality") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	itemCol.SetText( _("White balance") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	InsertColumn( i++, itemCol );

	// set the columns width
	CAppSettings& as = wxGetApp().GetAppSettings();
	for( i = 0; i < as.ListViewColumnsNumber(); i++ ) {
		int a = as.m_ListviewColWidth[i];
		SetColumnWidth( i, as.m_ListviewColWidth[i] );
	}

}

void CRightPaneList::StoreColumnsWidth() {

	CAppSettings& as = wxGetApp().GetAppSettings();
	for( int i = 0; i < as.ListViewColumnsNumber(); i++ ) {
		int a = GetColumnWidth( i );
		as.m_ListviewColWidth[i] = GetColumnWidth( i );
	}

}

bool CRightPaneList::IsAnythingSelected() {
	long item = -1;
	item = GetNextItem( item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
	return (item != -1); 
}

std::vector<int> CRightPaneList::GetSelectedItemsIndexes() {

	std::vector<int> indexes;
	long item = -1;
	for( ;; ) {
		item = GetNextItem( item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
		if( item == -1 ) break;
		indexes.push_back( item );
	}
	return indexes;
}

void CRightPaneList::SelectNone()
{
	std::vector<int> selectedIndexes = GetSelectedItemsIndexes();
	for( size_t k = 0; k < selectedIndexes.size(); k++ )
		SetItemState( selectedIndexes[k], 0, wxLIST_STATE_SELECTED );
}

void CRightPaneList::Select( int index )
{
	SetItemState( index, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
}

void CRightPaneList::SelectAll()
{
	for( int k = 0; k < GetItemCount(); k++ )
		Select( k );
}

/*!
 * wxEVT_CONTEXT_MENU event handler for ID_RIGHT_LISTCTRL
 */

void CRightPaneList::OnContextMenu( wxContextMenuEvent& event )
{
	wxPoint point = event.GetPosition();
	// If from keyboard
	if (point.x == -1 && point.y == -1) {
		wxSize size = GetSize();
		point.x = size.x / 2;
		point.y = size.y / 2;
	} else {
		point = ScreenToClient(point);
		// if there is a toolbar we need to correct the coordinates
		wxPoint origin = GetClientAreaOrigin();
		point += origin;
	}

	CAppSettings::CurrentView cv = wxGetApp().GetMainFrame()->GetCurrentView();

	wxMenu menu;
	if( cv == CAppSettings::cvFixed || cv == CAppSettings::cvRemovable ) {
		menu.Append( ID_ADD_VIRTUAL_FOLDER, _("Add To Virtual Folder") );
		menu.AppendSeparator();
	}

	menu.Append( ID_TOOLS_SHOW_IMAGE, _("Show Image") );
	menu.Append( ID_EDIT_OBJECT_INFORMATION, _("Image information\tCtrl-I") );

	// add external programs
	CExternalPrograms& ep = wxGetApp().GetAppSettings().m_ExternalPrograms;
	if( ep.size() > 0 ) {
		menu.AppendSeparator();
		CMainFrame* mf = dynamic_cast<CMainFrame*>( GetGrandParent() );
		mf->AppendExternalProgramsMenus( &menu, ep );
	}

	PopupMenu( &menu, point );
}

void CRightPaneList::ShowInfoDialog()
{
	wxASSERT( descriptionColumn >= 0 );

	std::vector<int> selectedIndexes = GetSelectedItemsIndexes();

	if( selectedIndexes.empty() ) return;

	if( selectedIndexes.size() == 1 ) {
		// only one item, show the image info dialog
		CFileData *fd = reinterpret_cast<CFileData *>( GetItemData(selectedIndexes[0]) );
		wxString orgDescr = fd->GetDescription();
		CImageInformationDialog dialog;
		dialog.SetFileData( fd );
		dialog.Create( GetGrandParent(), ID_CIMAGEINFORMATIONDIALOG, _("Image information") );
		if( dialog.ShowModal() != wxID_OK ) return;
		if( dialog.GetDescription() != orgDescr ) {
			// description has changed, update the list control
			SetItem( selectedIndexes[0], descriptionColumn, CUtils::NewLine2Space(dialog.GetDescription()) );
		}
	}
	else {
		// multiple items, show the object description dialog
		wxString orgDescr = wxEmptyString;
		CDialogObjectDescription dialog( GetGrandParent(), ID_CDIALOGOBJECTDESCRIPTION, _("Object description") );
		wxString objectName = _("Multiple items") + wxString::Format( wxT(" (%d)"), selectedIndexes.size() );
		dialog.SetObjectName( objectName );
		dialog.SetDescription( orgDescr );
		if( dialog.ShowModal() != wxID_OK ) return;

		wxString newDescr = dialog.GetDescription();
		if( newDescr == orgDescr) return;

		// set the new description
		for( int k = 0; k < static_cast<int>(selectedIndexes.size()); k++ ) {
			CFileData *fd = reinterpret_cast<CFileData *>( GetItemData(selectedIndexes[k]) );
			int fileID = fd->GetID();
			std::string sql = CSqlStrings::UpdatePhysicalFileDescription( fileID, newDescr );
			FSDB::Database db = wxGetApp().GetDatabase();
			db->ExecuteNoReturn(sql);
			fd->SetDescription( newDescr );
			SetItem( selectedIndexes[k], descriptionColumn, CUtils::NewLine2Space(newDescr) );
		}
	}
}

std::vector<CFileData*> CRightPaneList::GetSelectedItemsCFileData()
{
	std::vector<CFileData*> cfd;
	long item = -1;
	for( ;; ) {
		item = GetNextItem( item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
		if( item == -1 ) break;
		CFileData *fd = reinterpret_cast<CFileData *>( GetItemData(item) );
		cfd.push_back( fd );
	}
	return cfd;
}





/*!
 * wxEVT_SET_FOCUS event handler for ID_RIGHT_LISTCTRL
 */

void CRightPaneList::OnSetFocus( wxFocusEvent& WXUNUSED(event) )
{
	// send the event to notify that the window received the focus
	fsRightPaneFocusedEvent event( fsEVT_RIGHT_PANE_FOCUSED, GetId() );
	event.SetEventObject( this );
	GetEventHandler()->ProcessEvent( event );
}

