/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        serverfiledialog.h
// Purpose:     A dialog to specify a file on the server
// Author:      
// Modified by: 
// Created:     07/08/2008 22:01:04
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _SERVERFILEDIALOG_H_
#define _SERVERFILEDIALOG_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/valgen.h"
////@end includes

#include "window_position.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CSERVERFILEDIALOG 10023
#define ID_FILESERVER_CATALOG_NAME 10024
#define ID_FILESERVER_BROWSE 10025
#define SYMBOL_CSERVERFILEDIALOG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CSERVERFILEDIALOG_TITLE _("Server File Dialog")
#define SYMBOL_CSERVERFILEDIALOG_IDNAME ID_CSERVERFILEDIALOG
#define SYMBOL_CSERVERFILEDIALOG_SIZE wxSize(400, 300)
#define SYMBOL_CSERVERFILEDIALOG_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CServerFileDialog class declaration
 */

class CServerFileDialog: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CServerFileDialog )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CServerFileDialog();
    CServerFileDialog( wxWindow* parent, wxWindowID id = SYMBOL_CSERVERFILEDIALOG_IDNAME, const wxString& caption = SYMBOL_CSERVERFILEDIALOG_TITLE, const wxPoint& pos = SYMBOL_CSERVERFILEDIALOG_POSITION, const wxSize& size = SYMBOL_CSERVERFILEDIALOG_SIZE, long style = SYMBOL_CSERVERFILEDIALOG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CSERVERFILEDIALOG_IDNAME, const wxString& caption = SYMBOL_CSERVERFILEDIALOG_TITLE, const wxPoint& pos = SYMBOL_CSERVERFILEDIALOG_POSITION, const wxSize& size = SYMBOL_CSERVERFILEDIALOG_SIZE, long style = SYMBOL_CSERVERFILEDIALOG_STYLE );

    /// Destructor
    ~CServerFileDialog();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CServerFileDialog event handler declarations

    /// wxEVT_INIT_DIALOG event handler for ID_CSERVERFILEDIALOG
    void OnInitDialog( wxInitDialogEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_FILESERVER_BROWSE
    void OnFileserverBrowseClick( wxCommandEvent& event );

////@end CServerFileDialog event handler declarations

////@begin CServerFileDialog member function declarations

    wxString GetCatalogName() const { return m_CatalogName ; }
    void SetCatalogName(wxString value) { m_CatalogName = value ; }

    wxString GetAction() const { return m_Action ; }
    void SetAction(wxString value) { m_Action = value ; }

    bool GetShowBrowseButton() const { return m_ShowBrowseButton ; }
    void SetShowBrowseButton(bool value) { m_ShowBrowseButton = value ; }

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CServerFileDialog member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CServerFileDialog member variables
    wxStaticText* m_FirstTextRowCtrl;
    wxTextCtrl* m_CatalogNameCtrl;
    wxButton* m_BrowseCtrl;
private:
    wxString m_CatalogName;
    wxString m_Action; // "O" for OPEN, "N" for NEW, "B" for backup
    bool m_ShowBrowseButton; // if true the dialog will show a "Browse" button near the catalog name
////@end CServerFileDialog member variables

	CWindowPosition m_WindowPosition;
};

#endif
    // _SERVERFILEDIALOG_H_
