/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "sqlstrings.h"
#include "metadata.h"

std::string CSqlStrings::SelectVolumeByName( wxString volumeName, wxString volumeType ) {
	std::string retVal = "SELECT * FROM VOLUMES WHERE VOLUME_NAME = '" +
		CUtils::DBwx2std( volumeName ) + "' AND VOLUME_TYPE = '" + CUtils::DBwx2std( volumeType ) + "'";
	return retVal;
}

std::string CSqlStrings::SelectPathByName( wxString pathName, int volumeID, int fatherID ) {
	std::string retVal;
	if( fatherID == -1 ) {
		// we are looking for the root folder that has no name
		retVal = "SELECT * FROM PATHS WHERE VOLUME_ID = " + CUtils::long2stdstring(volumeID);
		retVal += " AND FATHER_ID IS NULL";
	}
	else {
		std::string pn = CUtils::DBwx2std( CUtils::ExpandSingleQuotes( pathName ) );
		retVal = "SELECT * FROM PATHS WHERE PATH_NAME = '" + pn + "' AND VOLUME_ID = " + CUtils::long2stdstring(volumeID);
		retVal += " AND FATHER_ID = " + CUtils::long2stdstring(fatherID);
	}
	return retVal;
}

std::string CSqlStrings::SelectVolumesList( wxString volumeType ) { 
	
	wxASSERT( volumeType == wxT("R") || volumeType == wxT("F") );

	return "SELECT * FROM VOLUMES WHERE VOLUME_TYPE = '" + 
		   CUtils::DBwx2std( volumeType ) + "' ORDER BY UPPER(VOLUME_NAME)"; 
}

std::string CSqlStrings::SelectVirtualPathsList( long fatherID ) {
	std::string sql;

	if( fatherID == -1 ) {
		sql = "SELECT * FROM VIRTUAL_PATHS WHERE FATHER_ID IS NULL ORDER BY UPPER(PATH_NAME)";
	}
	else {
		sql = "SELECT * FROM VIRTUAL_PATHS WHERE FATHER_ID = " + CUtils::long2stdstring(fatherID) + " ORDER BY UPPER(PATH_NAME)";
	}

	return sql;
}

std::string CSqlStrings::UpdateVolumeDescription( int volumeID, wxString newDescription ) {
	std::string sql;
	if( newDescription.empty() ) {
		sql = "UPDATE VOLUMES SET VOLUME_DESCRIPTION = NULL WHERE VOLUME_ID = " + CUtils::long2stdstring(volumeID);
	}
	else {
		std::string descr = CUtils::DBwx2std( CUtils::ExpandSingleQuotes( newDescription ) );
		sql = "UPDATE VOLUMES SET VOLUME_DESCRIPTION = '" + descr + "' WHERE VOLUME_ID = " + CUtils::long2stdstring(volumeID);
	}

	return sql;
}

std::string CSqlStrings::UpdatePhysicalPathDescription( int pathID, wxString newDescription ) {
	std::string sql;
	if( newDescription.empty() ) {
		sql = "UPDATE PATHS SET PATH_DESCRIPTION = NULL WHERE PATH_ID = " + CUtils::long2stdstring(pathID);
	}
	else {
		std::string descr = CUtils::DBwx2std( CUtils::ExpandSingleQuotes( newDescription ) );
		sql = "UPDATE PATHS SET PATH_DESCRIPTION = '" + descr + "' WHERE PATH_ID = " + CUtils::long2stdstring(pathID);
	}

	return sql;
}

std::string CSqlStrings::UpdateVirtualPathDescription( int pathID, wxString newDescription ) {
	std::string sql;
	if( newDescription.empty() ) {
		sql = "UPDATE VIRTUAL_PATHS SET PATH_DESCRIPTION = NULL WHERE PATH_ID = " + CUtils::long2stdstring(pathID);
	}
	else {
		std::string descr = CUtils::DBwx2std( CUtils::ExpandSingleQuotes( newDescription ) );
		sql = "UPDATE VIRTUAL_PATHS SET PATH_DESCRIPTION = '" + descr + "' WHERE PATH_ID = " + CUtils::long2stdstring(pathID);
	}

	return sql;
}

std::string CSqlStrings::UpdatePhysicalFileDescription( int fileID, wxString newDescription ) {
	std::string sql;
	if( newDescription.empty() ) {
		sql = "UPDATE FILES SET FILE_DESCRIPTION = NULL WHERE FILE_ID = " + CUtils::long2stdstring(fileID);
	}
	else {
		std::string descr = CUtils::DBwx2std( CUtils::ExpandSingleQuotes( newDescription ) );
		sql = "UPDATE FILES SET FILE_DESCRIPTION = '" + descr + "' WHERE FILE_ID = " + CUtils::long2stdstring(fileID);
	}

	return sql;
}

std::string CSqlStrings::SelectForSearchPane( wxString fileName, CUtils::TextSearchKind fnSearchKind, 
										wxString ext, wxString description, 
										CUtils::TextSearchKind descrSearchKind, 
										wxDateTime startDate, wxDateTime endDate,
										CUtils::SearchScope sscope,	int selectedFixedFolderID,
										int selectedRemovableFolderID, int selectedVirtualFolderID )
{
	std::string sql, wh;
	bool useWildcardsFilename, useWildcardsDescription;

	fileName = fileName.MakeUpper();
	ext = ext.MakeUpper();
	description = description.MakeUpper();

	// escape wildcard chars typed by the user
	// ...filename
	useWildcardsFilename = false;
	if( !fileName.empty() ) {
		switch( fnSearchKind ) {
			case CUtils::tskStartsWith:
				useWildcardsFilename = true;
				fileName = EscapeWildcards( fileName, wxT("/") );
				fileName = fileName + wxT("%");
				break;
			case CUtils::tskContains:
				useWildcardsFilename = true;
				fileName = EscapeWildcards( fileName, wxT("/") );
				fileName = wxT("%") + fileName + wxT("%");
				break;
		}
	}
	// ...description
	useWildcardsDescription = false;
	if( !description.empty() ) {
		switch( descrSearchKind ) {
			case CUtils::tskStartsWith:
				useWildcardsDescription = true;
				description = EscapeWildcards( description, wxT("/") );
				description = description + wxT("%");
				break;
			case CUtils::tskContains:
				useWildcardsDescription = true;
				description = EscapeWildcards( description, wxT("/") );
				description = wxT("%") + description + wxT("%");
				break;
		}
	}

	// create the WHERE clause
	wh = "";
	if( !fileName.empty() && useWildcardsFilename ) {
		if( !wh.empty() ) wh += " AND";
		wh = wh + " UPPER(FILE_NAME) LIKE '" + CUtils::DBwx2std(fileName) + "' ESCAPE '/'";
	}
	if( !fileName.empty() && !useWildcardsFilename ) {
		if( !wh.empty() ) wh += " AND";
		wh = wh + " UPPER(FILE_NAME) = '" + CUtils::DBwx2std(fileName) + "'";
	}
	if( !ext.empty() ) {
		if( !wh.empty() ) wh += " AND";
		wh = wh + " UPPER(FILE_EXT) = '" + CUtils::DBwx2std(CUtils::ExpandSingleQuotes(ext)) + "'";
	}
	if( !description.empty() && useWildcardsDescription ) {
		if( !wh.empty() ) wh += " AND";
		wh = wh + " UPPER(FILE_DESCRIPTION) LIKE '" + CUtils::DBwx2std(description) + "' ESCAPE '/'";
	}
	if( !description.empty() && !useWildcardsDescription ) {
		if( !wh.empty() ) wh += " AND";
		wh = wh + " UPPER(FILE_DESCRIPTION) = '" + CUtils::DBwx2std(description) + "'";
	}
	if( startDate.IsValid() ) {
		if( !wh.empty() ) wh += " AND";
		wh = wh + " IMAGE_DATETIME >= '" + CUtils::long2stdstring(startDate.GetYear(), 4) + "-" + CUtils::long2stdstring(startDate.GetMonth() + 1, 2) + "-" + CUtils::long2stdstring(startDate.GetDay(), 2) + "'";
	}
	if( endDate.IsValid() ) {
		if( !wh.empty() ) wh += " AND";
		wh = wh + " IMAGE_DATETIME <= '" + CUtils::long2stdstring(endDate.GetYear(), 4) + "-" + CUtils::long2stdstring(endDate.GetMonth() + 1, 2) + "-" + CUtils::long2stdstring(endDate.GetDay(), 2) + " 23:59:59'";
	}

	switch( sscope ) {
		case CUtils::ssAllFiles:
			sql = "SELECT FILES.*, THUMBNAILS.* FROM FILES LEFT OUTER JOIN THUMBNAILS ON FILES.FILE_ID = THUMBNAILS.FILE_ID_TH WHERE ";
			break;
		case CUtils::ssSelectedFixedLocationFolder:
			sql = "SELECT FILES.*, THUMBNAILS.* FROM FILES INNER JOIN PATHS_FILES ON FILES.FILE_ID = PATHS_FILES.FILE_ID_PF LEFT OUTER JOIN THUMBNAILS ON FILES.FILE_ID = THUMBNAILS.FILE_ID_TH WHERE PATH_ID_PF = " + CUtils::long2stdstring(selectedFixedFolderID) + " AND ";
			break;
		case CUtils::ssSelectedRemovableLocationFolder:
			sql = "SELECT FILES.*, THUMBNAILS.* FROM FILES INNER JOIN PATHS_FILES ON FILES.FILE_ID = PATHS_FILES.FILE_ID_PF LEFT OUTER JOIN THUMBNAILS ON FILES.FILE_ID = THUMBNAILS.FILE_ID_TH WHERE PATH_ID_PF = " + CUtils::long2stdstring(selectedRemovableFolderID) + " AND ";
			break;
		case CUtils::ssSelectedVirtualFolder:
			sql = "SELECT VIRTUAL_FILES.*, FILES.*, THUMBNAILS.* FROM (VIRTUAL_FILES INNER JOIN FILES ON VIRTUAL_FILES.PHYSICAL_FILE_ID = FILES.FILE_ID) LEFT OUTER JOIN THUMBNAILS ON FILES.FILE_ID = THUMBNAILS.FILE_ID_TH WHERE VIRTUAL_FILES.VIRTUAL_PATH_ID = " + CUtils::long2stdstring(selectedVirtualFolderID) + " AND ";
			break;
		default:
			wxASSERT( false );
	}

	sql = sql + wh + " ORDER BY FILES.IMAGE_DATETIME, UPPER(FILES.FILE_NAME)";
	
	return sql;
}


wxString CSqlStrings::EscapeWildcards( wxString txt, wxString escape ) {
	wxString retVal;

	retVal = CUtils::ExpandSingleQuotes( txt );
	retVal.Replace( wxT("_"), escape + wxT("_"), true );
	retVal.Replace( wxT("%"), escape + wxT("%"), true );

	return retVal;
}

std::string CSqlStrings::UpdateFileImageDatetime( int fileID, const wxDateTime &dt )
{
	std::string sql = "UPDATE FILES SET IMAGE_DATETIME = '" + CUtils::long2stdstring(dt.GetYear(), 4) + "-" + 
					   CUtils::long2stdstring(dt.GetMonth() + 1, 2) + "-" + 
					   CUtils::long2stdstring(dt.GetDay(), 2) + " " + CUtils::long2stdstring(dt.GetHour(), 2) + ":" +
					   CUtils::long2stdstring(dt.GetMinute(), 2) + ":" + CUtils::long2stdstring(dt.GetSecond(), 2) +
					   "' WHERE FILE_ID = " + CUtils::long2stdstring(fileID);
	return sql;
}

std::string CSqlStrings::SelectForTimelinePane( const wxDateTime &startDate, const wxDateTime &endDate, const int threshold )
{
    wxString stmp, wh;

    wh = "";
    if( startDate.IsValid() ) {
		if( !wh.empty() ) wh += " AND ";
		wh = wh + "IMAGE_DATETIME >= '" + CUtils::long2stdstring(startDate.GetYear(), 4) + "-" + CUtils::long2stdstring(startDate.GetMonth() + 1, 2) + "-" + CUtils::long2stdstring(startDate.GetDay(), 2) + "'" ;
	}
	if( endDate.IsValid() ) {
		if( !wh.empty() ) wh += " AND ";
		wh = wh + "IMAGE_DATETIME <= '" + CUtils::long2stdstring(endDate.GetYear(), 4) + "-" + CUtils::long2stdstring(endDate.GetMonth() + 1, 2) + "-" + CUtils::long2stdstring(endDate.GetDay(), 2) + " 23:59:59' ";
	}
    if( !wh.empty() ) {
        wh = "WHERE " + wh;
    }

    std::string sql = "SELECT CAST( CAST(IMAGE_DATETIME AS DATE) as TIMESTAMP ) AS IMAGE_DATE, COUNT(*) AS N_IMAGES FROM FILES ";
    if( !wh.empty() ) sql += wh;
    sql += "GROUP BY CAST( CAST(IMAGE_DATETIME AS DATE) as TIMESTAMP ) ";
    sql += "HAVING COUNT(*) > " + CUtils::long2stdstring(threshold) + " ";
    sql += "ORDER BY CAST( CAST(IMAGE_DATETIME AS DATE) as TIMESTAMP ) DESC";

    //std::string sql = "select substring(metadata.metadata_value from 1 for 10) as date_taken, count(*) ";
    //sql += "from files inner join metadata on files.file_id = metadata.file_id_md ";
    //sql += "where metadata.metadata_type = 3 ";
    //if( startDate.IsValid() ) {
    //    stmp = startDate.Format( "%YYYY:%mm:%dd" );
    //    sql += "and substring(metadata.metadata_value from 1 for 10) >= '" + stmp.ToStdString() + "' ";
    //}
    //if( endDate.IsValid() ) {
    //    stmp = endDate.Format( "%YYYY:%mm:%dd" );
    //    sql += "and substring(metadata.metadata_value from 1 for 10) <= '" + stmp.ToStdString() + "' ";
    //}
    //sql += "group by substring(metadata.metadata_value from 1 for 10) ";
    //if( threshold > 0 ) {
    //    sql += "having count(*) > " + CUtils::long2stdstring(threshold) + " ";
    //}
    //sql += "order by substring(metadata.metadata_value from 1 for 10) desc";

    return sql;
}

std::string CSqlStrings::SelectFilesByDate( const wxDateTime &dt )
{
    std::string sql = "SELECT FILES.*, THUMBNAILS.* FROM FILES LEFT OUTER JOIN THUMBNAILS ON FILES.FILE_ID = THUMBNAILS.FILE_ID_TH ";
    sql += "WHERE CAST(IMAGE_DATETIME AS DATE) = '" + CUtils::long2stdstring(dt.GetYear(), 4) + "-" + CUtils::long2stdstring(dt.GetMonth() + 1, 2) + "-" + CUtils::long2stdstring(dt.GetDay(), 2) + "' ";
    sql += "ORDER BY FILES.IMAGE_DATETIME, UPPER(FILES.FILE_NAME)";

    return sql;
}

std::string CSqlStrings::SelectCountFilesByDate( const wxDateTime &dt )
{
    std::string sql = "SELECT COUNT(FILES.FILE_ID) AS NROWS FROM FILES ";
    sql += "WHERE CAST(IMAGE_DATETIME AS DATE) = '" + CUtils::long2stdstring(dt.GetYear(), 4) + "-" + CUtils::long2stdstring(dt.GetMonth() + 1, 2) + "-" + CUtils::long2stdstring(dt.GetDay(), 2) + "'";

    return sql;
}


