/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        treectrl_physical.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     12/08/2008 17:28:28
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "wx/imaglist.h"
////@end includes

#include "vvvpapp.h"
#include "treectrl_physical.h"
#include "treectrl_physical_itemdata.h"
#include "utils.h"
#include "sqlstrings.h"
#include "left_right_panes_events.h"

#include <wx/filename.h>
#include <wx/imaglist.h>

////@begin XPM images
////@end XPM images

#include "graphics/folder_closed.xpm"
#include "graphics/folder_opened.xpm"
#include "graphics/removable.xpm"

/*!
 * CTreeCtrlPhysical type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CTreeCtrlPhysical, wxTreeCtrl )


/*!
 * CTreeCtrlPhysical event table definition
 */

BEGIN_EVENT_TABLE( CTreeCtrlPhysical, wxTreeCtrl )

////@begin CTreeCtrlPhysical event table entries
    EVT_TREE_SEL_CHANGED( wxID_ANY, CTreeCtrlPhysical::OnAnySelChanged )
    EVT_TREE_ITEM_EXPANDING( wxID_ANY, CTreeCtrlPhysical::OnAnyItemExpanding )
    EVT_TREE_ITEM_MENU( wxID_ANY, CTreeCtrlPhysical::OnAnyItemMenu )
    EVT_SET_FOCUS( CTreeCtrlPhysical::OnSetFocus )

////@end CTreeCtrlPhysical event table entries

END_EVENT_TABLE()


/*!
 * CTreeCtrlPhysical constructors
 */

CTreeCtrlPhysical::CTreeCtrlPhysical()
{
    Init();
}

CTreeCtrlPhysical::CTreeCtrlPhysical(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CTreePhysical creator
 */

bool CTreeCtrlPhysical::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style)
{
////@begin CTreeCtrlPhysical creation
    wxTreeCtrl::Create(parent, id, pos, size, style);
    CreateControls();
////@end CTreeCtrlPhysical creation
    return true;
}


/*!
 * CTreeCtrlPhysical destructor
 */

CTreeCtrlPhysical::~CTreeCtrlPhysical()
{
////@begin CTreeCtrlPhysical destruction
////@end CTreeCtrlPhysical destruction
}


/*!
 * Member initialisation
 */

void CTreeCtrlPhysical::Init()
{
////@begin CTreeCtrlPhysical member initialisation
////@end CTreeCtrlPhysical member initialisation
	m_IgnoreEvents = false;
}


/*!
 * Control creation for CTreePhysical
 */

void CTreeCtrlPhysical::CreateControls()
{    
////@begin CTreeCtrlPhysical content construction
////@end CTreeCtrlPhysical content construction
}


/*!
 * wxEVT_COMMAND_TREE_SEL_CHANGED event handler for ID_TREECTRL_PHYSICAL
 */

wxString CTreeCtrlPhysical::CreateVolumeLabel( const wxString& VolumeName, const wxString& VolumeDescription ) {
	wxString retVal;

	retVal = VolumeName;
	if( !VolumeDescription.empty() ) {
		wxString s = VolumeDescription.Left(20);
		s.Replace( wxT("\n"), wxT(" ") );
		retVal += wxT(" - ") + s;
	}
	return retVal;
}

void CTreeCtrlPhysical::LoadData() {

	wxImageList* iml = new wxImageList( 16, 16 );
	wxIcon ico(removable_xpm);
	iml->Add(ico);
	iml->Add(wxIcon(folder_closed_xpm));
	iml->Add(wxIcon(folder_opened_xpm));
	AssignImageList( iml );

	m_IgnoreEvents = true;
	DeleteAllItems();
	m_IgnoreEvents = false;

	// add a root item that will not be visible
	wxTreeItemId rootID = AddRoot( wxT("Root") );

	std::string sql = CSqlStrings::SelectVolumesList( volumeType );
	Database db = wxGetApp().GetDatabase();
	FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
	rs.Open( sql );
	while( !rs.IsEOF() ) {
		LoadVolume( rs["VOLUME_ID"].GetAsLargeInt(), CUtils::DBstd2wx(rs["VOLUME_NAME"]), CUtils::DBstd2wx(rs["VOLUME_DESCRIPTION"]), rootID );
		rs.MoveNext();
	}
	rs.Close();
}

void CTreeCtrlPhysical::LoadVolume( long volumeID, wxString volumeName, wxString volumeDescription, wxTreeItemId rootID ) {
	std::string sql;

	// first retrieves the id of the root folder for this volume
	sql = CSqlStrings::SelectVolumesPathID( volumeID );
	Database db = wxGetApp().GetDatabase();
	FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
	rs.Open( sql );
	long rootPathID = rs["PATH_ID"].GetAsLargeInt();
	rs.Close();

	// append the volume item
	wxTreeItemId volumeTreeID = AppendItem( rootID, CreateVolumeLabel(volumeName, volumeDescription), 0, -1, 
							new CTreePhysicalItemData(volumeName, volumeID, rootPathID, true, volumeDescription) );
	SetItemImage( volumeTreeID, 0, wxTreeItemIcon_Expanded );
	SetItemImage( volumeTreeID, 0, wxTreeItemIcon_SelectedExpanded );

	// append the first level of subfolders
	sql = CSqlStrings::SelectPathsList( volumeID, rootPathID );
	rs.Open( sql );
	while( !rs.IsEOF() ) {
		wxString name = CUtils::DBstd2wx( rs["PATH_NAME"] );
		// add the folder to the tree
		wxTreeItemId itemID = AppendItem( volumeTreeID, name, 1, 2, 
							new CTreePhysicalItemData(name, volumeID, rs["PATH_ID"].GetAsLargeInt(), false, CUtils::DBstd2wx(rs["PATH_DESCRIPTION"])) );
		// set the expanded images
		SetItemImage( itemID, 1, wxTreeItemIcon_Expanded );
		SetItemImage( itemID, 2, wxTreeItemIcon_SelectedExpanded );
		// add the subfolders
		LoadFolder( volumeID, itemID, rs["PATH_ID"].GetAsLargeInt() );

		rs.MoveNext();
	}
	rs.Close();
}

void CTreeCtrlPhysical::LoadFolder( long volumeID, wxTreeItemId fatherTreeID, long fatherID ) {

	Database db = wxGetApp().GetDatabase();
	FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
	std::string sql = CSqlStrings::SelectPathsList( volumeID, fatherID );
	rs.Open( sql );
	while( !rs.IsEOF() ) {
		wxString name = CUtils::DBstd2wx( rs["PATH_NAME"] );
		name = name.AfterLast( wxFileName::GetPathSeparator() );
		// add the folder to the tree
		wxTreeItemId itemID = AppendItem( fatherTreeID, name, 1, 2, 
							new CTreePhysicalItemData(name, volumeID, rs["PATH_ID"].GetAsLargeInt(), false, CUtils::DBstd2wx(rs["PATH_DESCRIPTION"])) );
		// set the expanded images
		SetItemImage( itemID, 1, wxTreeItemIcon_Expanded );
		SetItemImage( itemID, 2, wxTreeItemIcon_SelectedExpanded );


		rs.MoveNext();
	}
	rs.Close();
}


void CTreeCtrlPhysical::SetVolumeType( wxString volumeType ) {
	this->volumeType = volumeType;
}



/*!
 * wxEVT_COMMAND_TREE_SEL_CHANGED event handler for wxID_ANY
 */

void CTreeCtrlPhysical::OnAnySelChanged( wxTreeEvent& event )
{
	if( m_IgnoreEvents ) return;

	wxBusyCursor wait;
	
	wxTreeItemId itemID = event.GetItem();
    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);
	if( itemData != NULL ) {
		long pathID = itemData->GetPathID();
		GetMainFrame()->ShowFolderDataPhysical( pathID );
	}
}


/*!
 * wxEVT_COMMAND_TREE_ITEM_EXPANDING event handler for wxID_ANY
 */

void CTreeCtrlPhysical::OnAnyItemExpanding( wxTreeEvent& event )
{
	wxTreeItemId itemID = event.GetItem();
    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);

	if( itemData->IsVolume() || itemData->AlreadyOpened() ) return;

	// if this item has not been yet opened it loads the children of its children
	itemData->SetAlreadyOpened();

	wxBusyCursor wait;

	// loops over the children
	FSDB::Database db = wxGetApp().GetDatabase();
	db->TransactionStart();
	wxTreeItemIdValue cookie;
	wxTreeItemId childID = GetFirstChild( itemID, cookie );
	while( childID.IsOk() ) {
		// add the children of this child
	    CTreePhysicalItemData *childItemData = (CTreePhysicalItemData *) GetItemData(childID);
		LoadFolder( childItemData->GetVolumeID(), childID, childItemData->GetPathID() );
		childID = GetNextChild( itemID, cookie );
	}
	db->TransactionCommit();
}

int CTreeCtrlPhysical::GetSelectedVolumeID() {
	int retVal = -1;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothink is selected IsOk() returns true but itemData is NULL
		if( itemData->IsVolume() ) {
			retVal = itemData->GetVolumeID();
		}
	}

	return retVal;
}

int CTreeCtrlPhysical::GetSelectedPathID() {
	int retVal = -1;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothink is selected IsOk() returns true but itemData is NULL
		retVal = itemData->GetPathID();
	}

	return retVal;
}

wxString CTreeCtrlPhysical::GetSelectedItemName() {
	wxString retVal = wxEmptyString;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothink is selected IsOk() returns true but itemData is NULL
		retVal = itemData->GetDesc();
	}

	return retVal;
}

wxString CTreeCtrlPhysical::GetSelectedItemDescription() {
	wxString retVal = wxEmptyString;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothink is selected IsOk() returns true but itemData is NULL
		retVal = itemData->GetObjectDescription();
	}

	return retVal;
}

void CTreeCtrlPhysical::SetSelectedItemDescription( wxString descr ) {

	if( GetCount() <= 0 ) return;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);
		if( itemData == NULL ) return;		// generic TreeCtrl bug: if nothink is selected IsOk() returns true but itemData is NULL
		itemData->SetObjectDescription( descr );
		std::string sql;
		if( itemData->IsVolume() ) {
			sql = CSqlStrings::UpdateVolumeDescription( itemData->GetVolumeID(), descr );
			// change the text to reflect the new description
			SetItemText( itemID, CreateVolumeLabel( itemData->GetDesc(), descr ) );
		}
		else {
			sql = CSqlStrings::UpdatePhysicalPathDescription( itemData->GetPathID(), descr );
		}
		wxGetApp().GetDatabase()->ExecuteNoReturn( sql );
	}
}

void CTreeCtrlPhysical::RemoveSelectedItem() {

	if( GetCount() <= 0 ) return;
	wxTreeItemId itemID = GetSelection();
	Delete( itemID );
}

bool CTreeCtrlPhysical::IsNodeSelected() {
	bool isSelected = (GetCount() > 0);
	if( isSelected ) {
		wxTreeItemId item = GetSelection();
		isSelected = item.IsOk();
	}
	return isSelected;
}


/*!
 * wxEVT_COMMAND_TREE_ITEM_MENU event handler for wxID_ANY
 */

void CTreeCtrlPhysical::OnAnyItemMenu( wxTreeEvent& event )
{
	wxTreeItemId item = event.GetItem();
	if( !item.IsOk() ) return;

	if( item != GetSelection() ) {
		// do not re-select the same item to avoid delays
		SelectItem( item );
	}

	wxPoint pt = event.GetPoint();
    // if there is a toolbar we need to correct the coordinates
	wxPoint origin = GetClientAreaOrigin();
    pt += origin;

	wxMenu menu;
	menu.Append( ID_ADD_VIRTUAL_FOLDER, _("Add To Virtual Folder") );
	menu.AppendSeparator();
	menu.Append( ID_EDIT_OBJECT_INFORMATION, _("Object Information...") );
	menu.Append( ID_EDIT_DELETE, _("Delete") );

	PopupMenu( &menu, pt );

	event.Skip();
}


/*!
 * wxEVT_SET_FOCUS event handler for wxID_ANY
 */

void CTreeCtrlPhysical::OnSetFocus( wxFocusEvent& WXUNUSED(event) )
{
	// send the event to notify that the window received the focus
	fsLeftPaneFocusedEvent event( fsEVT_LEFT_PANE_FOCUSED, GetId() );
	event.SetEventObject( this );
	GetEventHandler()->ProcessEvent( event );
}

int CTreeCtrlPhysical::GetVolumeIDFromSelectedPath()
{
	int retVal = -1;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreePhysicalItemData *itemData = (CTreePhysicalItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothink is selected IsOk() returns true but itemData is NULL
		retVal = itemData->GetVolumeID();
	}

	return retVal;
}


