/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __TREE_PHYSICAL_ITEMDATA_H__
#define __TREE_PHYSICAL_ITEMDATA_H__

// this class implements a custom ItemData class to store information about physical tree control items

#include "wx/treectrl.h"

class CTreePhysicalItemData : public wxTreeItemData {
public:
	// constructor used for physical folders tree
	CTreePhysicalItemData( const wxString& desc, long VolumeID, long PathID, bool IsVolume, wxString ObjectDescription ) : 
	                m_desc(desc), m_VolumeID(VolumeID), m_PathID(PathID), m_IsVolume(IsVolume), m_AlreadyOpened(false), m_ObjectDescription(ObjectDescription) {}
	const wxString& GetDesc() const { return m_desc; }
	void SetDesc( const wxString& Descr ) { m_desc = Descr; }
	long GetVolumeID(void) { return m_VolumeID; }
	long GetPathID(void) { return m_PathID; }
	bool IsVolume(void) { return m_IsVolume; }
	bool AlreadyOpened(void) { return m_AlreadyOpened; }
	void SetAlreadyOpened(void) { m_AlreadyOpened = true; }
	const wxString& GetObjectDescription() const { return m_ObjectDescription; }
	void SetObjectDescription( const wxString& ObjDescr ) { m_ObjectDescription = ObjDescr; }
private:
	wxString m_desc;
	long m_VolumeID, m_PathID;
	bool m_IsVolume;	// true if this node is a volume and not a folder
	bool m_AlreadyOpened;	// true if this node has already been opened by the user
	wxString m_ObjectDescription;	// object description
};

#endif
