/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "vvvpapp.h"
#include "treectrl_virtual.h"
#include "treectrl_virtual_itemdata.h"
#include "utils.h"
#include "sqlstrings.h"
#include "left_right_panes_events.h"

#include <wx/filename.h>
#include <wx/imaglist.h>

////@begin XPM images
////@end XPM images

#include "graphics/folder_closed.xpm"
#include "graphics/folder_opened.xpm"
#include "graphics/removable.xpm"

/*!
 * CTreeCtrlVirtual type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CTreeCtrlVirtual, wxTreeCtrl )


/*!
 * CTreeCtrlVirtual event table definition
 */

BEGIN_EVENT_TABLE( CTreeCtrlVirtual, wxTreeCtrl )

////@begin CTreeCtrlVirtual event table entries
    EVT_TREE_SEL_CHANGED( wxID_ANY, CTreeCtrlVirtual::OnAnySelChanged )
    EVT_TREE_ITEM_EXPANDING( wxID_ANY, CTreeCtrlVirtual::OnAnyItemExpanding )
    EVT_TREE_ITEM_MENU( wxID_ANY, CTreeCtrlVirtual::OnAnyItemMenu )

////@end CTreeCtrlVirtual event table entries

    EVT_SET_FOCUS( CTreeCtrlVirtual::OnSetFocus )

END_EVENT_TABLE()


/*!
 * CTreeCtrlVirtual constructors
 */

CTreeCtrlVirtual::CTreeCtrlVirtual()
{
    Init();
}

CTreeCtrlVirtual::CTreeCtrlVirtual(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CTreeCtrlVirtual creator
 */

bool CTreeCtrlVirtual::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style)
{
////@begin CTreeCtrlVirtual creation
    wxTreeCtrl::Create(parent, id, pos, size, style);
    CreateControls();
////@end CTreeCtrlVirtual creation
    return true;
}


/*!
 * CTreeCtrlVirtual destructor
 */

CTreeCtrlVirtual::~CTreeCtrlVirtual()
{
////@begin CTreeCtrlVirtual destruction
////@end CTreeCtrlVirtual destruction
}


/*!
 * Member initialisation
 */

void CTreeCtrlVirtual::Init()
{
////@begin CTreeCtrlVirtual member initialisation
////@end CTreeCtrlVirtual member initialisation
	m_IgnoreEvents = false;
	m_updateRightPane = false;
}


/*!
 * Control creation for CTreeCtrlVirtual
 */

void CTreeCtrlVirtual::CreateControls()
{    
////@begin CTreeCtrlVirtual content construction
////@end CTreeCtrlVirtual content construction
}


wxString CTreeCtrlVirtual::CreateVolumeLabel( const wxString& VolumeName, const wxString& VolumeDescription ) {
	wxString retVal;

	retVal = VolumeName;
	if( !VolumeDescription.empty() ) {
		wxString s = VolumeDescription.Left(20);
		s.Replace( wxT("\n"), wxT(" ") );
		retVal += wxT(" - ") + s;
	}
	return retVal;
}

void CTreeCtrlVirtual::LoadData() {

	wxImageList* iml = new wxImageList( 16, 16 );
	wxIcon ico(removable_xpm);
	iml->Add(ico);
	iml->Add(wxIcon(folder_closed_xpm));
	iml->Add(wxIcon(folder_opened_xpm));
	AssignImageList( iml );

	m_IgnoreEvents = true;
	DeleteAllItems();
	m_IgnoreEvents = false;

	// add a root item that will not be visible
	wxTreeItemId rootID = AddRoot( wxT("Root") );

	std::string sql = CSqlStrings::SelectVirtualPathsList( -1 );
	Database db = wxGetApp().GetDatabase();
	FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
	rs.Open( sql );
	while( !rs.IsEOF() ) {
		wxString name = CUtils::DBstd2wx( rs["PATH_NAME"] );
		// add the folder to the tree
		wxTreeItemId itemID = AppendItem( rootID, name, 1, 2, 
			new CTreeVirtualItemData(name, rs["PATH_ID"].GetAsLargeInt(), CUtils::DBstd2wx(rs["PATH_DESCRIPTION"]), rs["PHYS_PATH_ID"].IsNull() ? -1 : static_cast<int>(rs["PHYS_PATH_ID"]) ) );
		
		if( rs["PHYS_PATH_ID"].IsNull() ) {
			// this vistual path has been created by the user: it is not a physical path assigned to a virtual path
			// show it in a different colour
			SetItemTextColour( itemID, *wxBLUE );
		}

		// set the expanded images
		SetItemImage( itemID, 1, wxTreeItemIcon_Expanded );
		SetItemImage( itemID, 2, wxTreeItemIcon_SelectedExpanded );
		// add the subfolders
		LoadFolder( itemID, rs["PATH_ID"].GetAsLargeInt() );

		rs.MoveNext();
	}
	rs.Close();
}

void CTreeCtrlVirtual::LoadFolder( wxTreeItemId fatherTreeID, long fatherID ) {

	Database db = wxGetApp().GetDatabase();
	FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
	std::string sql = CSqlStrings::SelectVirtualPathsList( fatherID );
	rs.Open( sql );
	while( !rs.IsEOF() ) {
		wxString name = CUtils::DBstd2wx( rs["PATH_NAME"] );
		name = name.AfterLast( wxFileName::GetPathSeparator() );
		// add the folder to the tree
		wxTreeItemId itemID = AppendItem( fatherTreeID, name, 1, 2, 
							new CTreeVirtualItemData(name, rs["PATH_ID"].GetAsLargeInt(), CUtils::DBstd2wx(rs["PATH_DESCRIPTION"]), rs["PHYS_PATH_ID"].IsNull() ? -1 : static_cast<int>(rs["PHYS_PATH_ID"]) ) );

		if( rs["PHYS_PATH_ID"].IsNull() ) {
			// this vistual path has been created by the user: it is not a physical path assigned to a virtual path
			// show it in a different colour
			SetItemTextColour( itemID, *wxBLUE );
		}

		// set the expanded images
		SetItemImage( itemID, 1, wxTreeItemIcon_Expanded );
		SetItemImage( itemID, 2, wxTreeItemIcon_SelectedExpanded );

		rs.MoveNext();
	}
	rs.Close();
}



/*!
 * wxEVT_COMMAND_TREE_SEL_CHANGED event handler for wxID_ANY
 */

void CTreeCtrlVirtual::OnAnySelChanged( wxTreeEvent& event )
{
	if( m_IgnoreEvents ) return;

	wxBusyCursor wait;
	
	wxTreeItemId itemID = event.GetItem();
    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) GetItemData(itemID);
	if( itemData != NULL && m_updateRightPane ) {
		long pathID = itemData->GetPathID();
		GetMainFrame()->ShowFolderDataVirtual( pathID );
	}
}


/*!
 * wxEVT_COMMAND_TREE_ITEM_EXPANDING event handler for wxID_ANY
 */

void CTreeCtrlVirtual::OnAnyItemExpanding( wxTreeEvent& event )
{
	wxTreeItemId itemID = event.GetItem();
    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) GetItemData(itemID);

	if( itemData->AlreadyOpened() ) return;

	// if this item has not been yet opened it loads the children of its children
	itemData->SetAlreadyOpened();

	wxBusyCursor wait;

	// loops over the children
	FSDB::Database db = wxGetApp().GetDatabase();
	db->TransactionStart();
	wxTreeItemIdValue cookie;
	wxTreeItemId childID = GetFirstChild( itemID, cookie );
	while( childID.IsOk() ) {
		// add the children of this child
	    CTreeVirtualItemData *childItemData = (CTreeVirtualItemData *) GetItemData(childID);
		LoadFolder( childID, childItemData->GetPathID() );
		childID = GetNextChild( itemID, cookie );
	}
	db->TransactionCommit();
}

wxString CTreeCtrlVirtual::GetSelectedItemName() {
	wxString retVal = wxEmptyString;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothing is selected IsOk() returns true but itemData is NULL
		retVal = itemData->GetDesc();
	}

	return retVal;
}

void CTreeCtrlVirtual::SetSelectedItemName( wxString newName )
{
	if( GetCount() <= 0 ) return;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) GetItemData(itemID);
		if( itemData == NULL ) return;		// generic TreeCtrl bug: if nothing is selected IsOk() returns true but itemData is NULL
		SetItemText( itemID, newName );
		itemData->SetDesc( newName );
		std::string sql = CSqlStrings::UpdateVirtualPathName( itemData->GetPathID(), newName );
		wxGetApp().GetDatabase()->ExecuteNoReturn( sql );
	}
}


wxString CTreeCtrlVirtual::GetSelectedItemDescription() {
	wxString retVal = wxEmptyString;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothing is selected IsOk() returns true but itemData is NULL
		retVal = itemData->GetObjectDescription();
	}

	return retVal;
}

void CTreeCtrlVirtual::SetSelectedItemDescription( wxString descr ) {

	if( GetCount() <= 0 ) return;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) GetItemData(itemID);
		if( itemData == NULL ) return;		// generic TreeCtrl bug: if nothing is selected IsOk() returns true but itemData is NULL
		itemData->SetObjectDescription( descr );
		std::string sql = CSqlStrings::UpdateVirtualPathDescription( itemData->GetPathID(), descr );
		wxGetApp().GetDatabase()->ExecuteNoReturn( sql );
	}
}

void CTreeCtrlVirtual::RemoveSelectedItem() {

	if( GetCount() <= 0 ) return;
	wxTreeItemId itemID = GetSelection();
	Delete( itemID );
}

bool CTreeCtrlVirtual::IsNodeSelected() {
	bool isSelected = (GetCount() > 0);
	if( isSelected ) {
		wxTreeItemId item = GetSelection();
		isSelected = item.IsOk();
	}
	return isSelected;
}

int CTreeCtrlVirtual::GetSelectedPathID() {
	int retVal = -1;

	if( GetCount() <= 0 ) return retVal;

	wxTreeItemId itemID = GetSelection();
	if( itemID.IsOk() ) {
	    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) GetItemData(itemID);
		if( itemData == NULL ) return retVal;		// generic TreeCtrl bug: if nothink is selected IsOk() returns true but itemData is NULL
		retVal = itemData->GetPathID();
	}

	return retVal;
}

/*!
 * wxEVT_COMMAND_TREE_ITEM_MENU event handler for wxID_ANY
 */

void CTreeCtrlVirtual::OnAnyItemMenu( wxTreeEvent& event )
{
	wxTreeItemId item = event.GetItem();
	if( !item.IsOk() ) return;

	if( item != GetSelection() ) {
		// do not re-select the same item to avoid delays
		SelectItem( item );
	}

	wxPoint pt = event.GetPoint();
    // if there is a toolbar we need to correct the coordinates
	wxPoint origin = GetClientAreaOrigin();
    pt += origin;

	wxMenu menu;
	menu.Append( ID_NEW_VIRTUAL_ROOT_FOLDER, _("New Root Folder") );
	if( item.IsOk() ) {
		// only if we have a selected item
		menu.Append( ID_NEW_VIRTUAL_SUBFOLDER, _("New Subfolder") );
		menu.AppendSeparator();
		menu.Append( ID_EDIT_OBJECT_INFORMATION, _("Object Information...") );
		menu.Append( ID_EDIT_RENAME, _("Rename") );
		menu.Append( ID_EDIT_DELETE, _("Delete") );
	}

	PopupMenu( &menu, pt );

	event.Skip();
}

void CTreeCtrlVirtual::OnSetFocus( wxFocusEvent& WXUNUSED(event) )
{
	// send the event to notify that the window received the focus
	fsLeftPaneFocusedEvent event( fsEVT_LEFT_PANE_FOCUSED, GetId() );
	event.SetEventObject( this );
	GetEventHandler()->ProcessEvent( event );
}

