/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __TREE_VIRTUAL_ITEMDATA_H__
#define __TREE_VIRTUAL_ITEMDATA_H__

// this class implements a custom ItemData class to store information about virtual tree control items

#include "wx/treectrl.h"

class CTreeVirtualItemData : public wxTreeItemData {
public:
	// constructor used for virtual folders tree
	CTreeVirtualItemData( const wxString& desc, long PathID, wxString ObjectDescription, long PhysPathID ) : 
	                m_desc(desc), m_PathID(PathID), m_AlreadyOpened(false), m_ObjectDescription(ObjectDescription), m_PhysPathID(PhysPathID) {}
	const wxString& GetDesc() const { return m_desc; }
	void SetDesc( const wxString& Descr ) { m_desc = Descr; }
	long GetPathID(void) { return m_PathID; }
	long GetPhysPathID(void) { return m_PhysPathID; }
	bool AlreadyOpened(void) { return m_AlreadyOpened; }
	void SetAlreadyOpened(void) { m_AlreadyOpened = true; }
	const wxString& GetObjectDescription() const { return m_ObjectDescription; }
	void SetObjectDescription( const wxString& ObjDescr ) { m_ObjectDescription = ObjDescr; }
private:
	wxString m_desc;
	long m_PathID;
	long m_PhysPathID;	// physical path id. -1 meand NULL;
	bool m_AlreadyOpened;	// true if this node has already been opened by the user
	wxString m_ObjectDescription;	// object description
};

#endif
