/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        vvvpapp.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     30/07/2008 21:52:10
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _VVVPAPP_H_
#define _VVVPAPP_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/image.h"
#include "mainframe.h"
////@end includes

#include "fixed_locations.h"

#include "fsdb.h"

#include "wx/help.h"
#include "wx/html/helpctrl.h"
#include "wx/fs_zip.h"

/*!
 * Forward declarations
 */

class CAppSettings;

////@begin forward declarations
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
////@end control identifiers

/*!
 * CVvvPApp class declaration
 */

class CVvvPApp: public wxApp
{    
    DECLARE_CLASS( CVvvPApp )
    DECLARE_EVENT_TABLE()

public:
    /// Constructor
    CVvvPApp();

    void Init();

    /// Initialises the application
    virtual bool OnInit();

    /// Called on exit
    virtual int OnExit();

	// derived to add exception handling
	virtual int OnRun();
	virtual bool OnExceptionInMainLoop();
	virtual void HandleEvent(wxEvtHandler* handler, wxEventFunction func, wxEvent& event) const;

////@begin CVvvPApp event handler declarations

////@end CVvvPApp event handler declarations

////@begin CVvvPApp member function declarations

////@end CVvvPApp member function declarations

////@begin CVvvPApp member variables
////@end CVvvPApp member variables

	/// Open a database.
	/// Returns true if OK, returns false if not opened.
	bool OpenDatabase( wxString fileName, int expectedVersion );
	
	FSDB::Database GetDatabase() { return m_db; }

	/// return true if the program is connected to a database
	bool IsDatabaseConnected() { return m_DbConnected; }

	// update the database statistics for the physical views (currently removable and fixed)
	void UpdateDBStatisticsPhysical();

	// update the database statistics for the virtual view
	void UpdateDBStatisticsVirtual();

	CAppSettings& GetAppSettings() { return m_AppSettings; }

	CFixedLocations& GetFixedLocations() { return m_FixedLocations; }

	// returns the name of the catalog passed in the command line (or "" if none)
	wxString GetParameterCatalog() { return m_CatalogName; }

	// return the name of the settings file (or "" if none). used for the portable version
	wxString GetSettingsFileName() { return m_SettingsFileName; }

	// return the name of the default data folder (or "" if none). used for the portable version
	wxString GetDefaultDataFolder() { return m_DefaultDataFolder; }

	wxHelpController *GetHelpController() { return m_HelpController; }

	CMainFrame* GetMainFrame() { return mainWindow; }

private:

	/// the database used by the application
	FSDB::Database m_db;

	/// true if a database is connected
	bool m_DbConnected;

	/// application settings
	CAppSettings m_AppSettings;

	// fixed locations for the current catalog
	CFixedLocations m_FixedLocations;

    wxLocale m_locale;

	// name of the catalog passed in the command line
	wxString m_CatalogName;

	// name of the file used to store the program's settings. Used only for the portable version.
	wxString m_SettingsFileName;

	// name of the default folder when creating a new catalog. Used for the portable version.
	wxString m_DefaultDataFolder;

	// help controller
	wxHelpController *m_HelpController;

	// pointer to the main program frame
	CMainFrame* mainWindow;

};

/*!
 * Application instance declaration 
 */

////@begin declare app
DECLARE_APP(CVvvPApp)
////@end declare app


#endif
    // _VVVPAPP_H_
