/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// Windows specific code
// compile only for the Windows version

#include <windows.h>
#include <io.h>
#include <wx/app.h>
#include <wx/dir.h>
#include <wx/filename.h>

#include "windows_specific.h"
#include "compare_folders.h"

// return the name of a volume specified in a way like "C:\"
// return an empty string in case of error
wxString GetVolumeName( wxString volume ) {
	TCHAR vn[50];
	DWORD mcl, fsf;
	wxString retVal;

	if( GetVolumeInformation( volume.c_str(), vn, 49, NULL, &mcl, &fsf, NULL, 0 ) )
		retVal = vn;
	else
		retVal = wxEmptyString;
	
	return retVal;
}

wxString GetSystemDirectory_wx()
{
	TCHAR buf[MAX_PATH + 50];
	wxString retVal = wxEmptyString;

	if( GetSystemDirectory(buf, MAX_PATH) > 0 ) {
		retVal = buf;
		if( !retVal.EndsWith(wxT("\\")) ) {
			retVal += wxT("\\");
		}
	}
	return retVal;
}


// return the datetime and size of the current file
void GetFileDateSizeWindows( WIN32_FIND_DATA &ffd, wxDateTime &FileDateTime, wxLongLong &FileSize ) {
	SYSTEMTIME stLocal;
	FILETIME ft;
	FileTimeToLocalFileTime( &ffd.ftLastWriteTime, &ft );
	FileTimeToSystemTime(&ft, &stLocal);
	FileDateTime.Set( stLocal.wDay, (wxDateTime::Month) (stLocal.wMonth - 1), stLocal.wYear, stLocal.wHour, stLocal.wMinute, stLocal.wSecond, stLocal.wMilliseconds );
	FileSize = ( ((__int64)ffd.nFileSizeHigh) << 32 ) + ffd.nFileSizeLow;
}

void CCompareFolders::ReadAllFilesWindows( wxString basePath, wxString path, std::map<wxString, CFileData> &files, bool recurse ) {

	wxString fileName;
	wxString winPath;	// path to be used for Windows-specific calls

	// builds the physical path for this folder
	wxString physPath = basePath + path;

	winPath = physPath;
	if( !winPath.EndsWith(wxT("\\")) )
		winPath += wxT("\\");
	winPath += wxT("*.*");

	// read all the files in the current folder
	WIN32_FIND_DATA ffd;
	HANDLE sh = FindFirstFile( winPath.c_str(), &ffd );
	if( sh != INVALID_HANDLE_VALUE ) {
		do {
			if( !(ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) ) {
				// exclude folders
				wxString name;
				if( !path.empty() )
					name = path + wxFileName::GetPathSeparator();
				name += ffd.cFileName;

				CFileData *fd = new CFileData();
				wxDateTime fTime;
				wxLongLong fSize;
				GetFileDateSizeWindows( ffd, fTime, fSize );
				fd->DateTime = fTime;
				fd->FileSize = fSize;

				files.insert( FDPair(name, *fd) );
				delete fd;
			}
		} while( FindNextFile(sh, &ffd) );
	}
	FindClose( sh );

	if( !recurse ) return;

	// recurse in the subfolders
	_tfinddata_t c_file;
	intptr_t hFile = _tfindfirst( winPath.c_str(), &c_file );
	if( hFile != -1L ) {
		do {
			if( (c_file.attrib & _A_SUBDIR) && (_tcscmp(c_file.name, wxT(".")) != 0) && (_tcscmp(c_file.name, wxT("..")) != 0) ) {
				wxString newPath;
				if( !path.empty() )
					newPath = path + wxFileName::GetPathSeparator();
				newPath += c_file.name;

				ReadAllFilesWindows( basePath, newPath, files, recurse );
			}
		} while( _tfindnext( hFile, &c_file ) == 0 );
	}
	_findclose( hFile );
}


